/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
/************************************************************************
** MODULE INFORMATION*
**********************
**     FILE     NAME:       hostm.c
**     SYSTEM   NAME:       host
**     ORIGINAL AUTHOR(S):  M.F.B. de Greeve
**     VERSION  NUMBER:     1.0
**     CREATION DATE:       1992/9/25
**
** DESCRIPTION: host group of the RMON MIB
**              
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:    $Revision$
** WORKFILE:    $Workfile$
** LOGINFO:     $Log$
*************************************************************************/


#include <dnpap.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <message.h>
#include <mibsup.h>
#include <mac.h>

#include "hoste.h"
#include "hostc.h"
#include "host.h"


#define CONTROLINDEXSIZE   1
#define HOSTINDEXSIZE      7
#define TIMEINDEXSIZE      2


static MIB_LOCAL    *HostControlInstance = NULL;


static BOOLEAN RmonNext (SNMP_OBJECT *obj, MIB_LOCAL **local, WORD idlen, WORD indexsize, HOST **elem);

static HOST* HostGetFirstNext(HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first);



BOOLEAN HostMInit(VOID)
{
    MessageConfig(HOST_ERROR, "Host");
    return HostConfigInit();
}


BOOLEAN RmonNext (SNMP_OBJECT *obj, MIB_LOCAL **local, WORD idlen, WORD indexsize, HOST **elem)
{
BOOLEAN first = FALSE;

    if (*local == NULL || indexsize < 1)
        return FALSE;

    if (obj->IdLen < idlen + indexsize)
    {
        switch (indexsize)
        {
        case HOSTINDEXSIZE:
            while (*local != NULL && (*elem = HostGetFirstNext((HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + HOSTINDEXSIZE;
            break;
        case TIMEINDEXSIZE:
            while (*local != NULL && ((HOST_CONTROL*)(*local)->Data)->TableSize == 0)
                *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->Id[idlen+1] = 1;
            obj->IdLen = idlen + TIMEINDEXSIZE;
            *elem = HostTimeSearch((HOST_CONTROL*)(*local)->Data, obj, idlen);
            break;
        case CONTROLINDEXSIZE:
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + CONTROLINDEXSIZE;
            break;
        }
        return TRUE;
    }
    else
    {
        switch (indexsize)
        {            
        case HOSTINDEXSIZE:
            while (*local != NULL && (*elem = HostGetFirstNext((HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + HOSTINDEXSIZE;
            break;
        case TIMEINDEXSIZE:
            if (obj->Id[idlen+1] < ((HOST_CONTROL*)(*local)->Data)->TableSize)
            {
                obj->Id[idlen] = (*local)->Index;
                obj->Id[idlen+1]++;
                obj->IdLen = idlen + TIMEINDEXSIZE;
            	*elem = HostTimeSearch((HOST_CONTROL*)(*local)->Data, obj, idlen);
                break;
            }
            *local = (*local)->Next;
            while (*local != NULL && ((HOST_CONTROL*)(*local)->Data)->TableSize == 0)
                *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->Id[idlen+1] = 1;
            obj->IdLen = idlen + TIMEINDEXSIZE;
            *elem = HostTimeSearch((HOST_CONTROL*)(*local)->Data, obj, idlen);
            break;
        case CONTROLINDEXSIZE:
			/* only go to next collector if requested index was higher or equal */
			if (obj->Id[idlen] >= (*local)->Index)
	            *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + CONTROLINDEXSIZE;
            break;
        }
        return TRUE;
    }
    return FALSE;
}


HOST *HostGetFirstNext(HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first)
{
HOST host, *p;
INT i, len;

    if (hostcontrol->TableSize == 0)
        return NULL;

    if (first == TRUE)
    {           
        p = hostcontrol->LexList;
        for (i = 0; i < HOST_SZEADDR; i++)
            obj->Id[idlen+1+i] = (LONG)p->Address[i];
        obj->IdLen = idlen + HOSTINDEXSIZE;
        return p;
    }

    len = obj->IdLen-idlen-1;
    if (len < 0)
        len = 0;

    for (i = 0; i < len; i++)
        host.Address[i] = (BYTE)obj->Id[idlen+1+i];

    p = HostHashSearch(hostcontrol, &host, len);

    if (len == HOST_SZEADDR && p != NULL)
        p = p->LexNext;

    if (p != NULL)
    {
        for (i = 0; i < HOST_SZEADDR; i++)
            obj->Id[idlen+1+i] = (LONG)p->Address[i];
        obj->IdLen = idlen + HOSTINDEXSIZE;
    }
    return p;
}


WORD hostControlIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostControlDataSource (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
HOST_CONTROL *data;
MAC_IFACE    *iface;
LONG         IfaceOid[] = {1,3,6,1,2,1,2,2,1,1,1};
WORD         IfaceOidLen = sizeof(IfaceOid)/sizeof(LONG);
LONG         ChannelOid[] = {1,3,6,1,2,1,16,7,2,1,1,1};
WORD         ChannelOidLen = sizeof(ChannelOid)/sizeof(LONG);

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {           
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (HOST_CONTROL*) local->Data;
        memcpy (Obj->Syntax.BufInt, data->Source, data->SourceLen*sizeof(data->Source[0]));
        Obj->SyntaxLen = data->SourceLen;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (HOST_CONTROL*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (IfaceOidLen == Obj->SyntaxLen &&
            memcmp(IfaceOid, Obj->Syntax.BufInt, (IfaceOidLen-1)*sizeof(IfaceOid[0])) == 0)
        {
            data->Channel = FALSE;
            if ((iface = MacIfaceGet((WORD) Obj->Syntax.BufInt[Obj->SyntaxLen-1])) == NULL)
                return SNMP_BADVALUE;
            if (iface->type != MAC_TYPE_ETHERNET_CSMACD && iface->type != MAC_TYPE_88023_CSMACD)
           	    return SNMP_BADVALUE;
            memcpy(data->Source, Obj->Syntax.BufInt, Obj->SyntaxLen*sizeof(data->Source[0]));
            data->SourceLen = Obj->SyntaxLen;
            data->Iface = iface;
            return SNMP_NOERROR;
        }
        else
        if (ChannelOidLen == Obj->SyntaxLen &&
            memcmp(ChannelOid, Obj->Syntax.BufInt, (ChannelOidLen-1)*sizeof(ChannelOid[0])) == 0)
        {                         
            data->Channel = TRUE;
            memcpy(data->Source, Obj->Syntax.BufInt, Obj->SyntaxLen*sizeof(data->Source[0]));
            data->SourceLen = Obj->SyntaxLen;
            return SNMP_NOERROR;
        }
        return SNMP_BADVALUE;
    }
    return SNMP_GENERROR;
}


WORD hostControlTableSize (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->TableSize;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostControlLastDeleteTime (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (HOST_CONTROL*) local->Data;
        Obj->Syntax.LngUns = data->LastDeleteTime;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostControlOwner (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (HOST_CONTROL*) local->Data;
        memcpy (Obj->Syntax.BufChr, data->Owner, data->OwnerLen);
        Obj->SyntaxLen = data->OwnerLen;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (HOST_CONTROL*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        memcpy (data->Owner, Obj->Syntax.BufChr, Obj->SyntaxLen);
        data->OwnerLen = Obj->SyntaxLen;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostControlStatus (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
HOST_CONTROL *data;

    local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE);

    switch (Obj->Request)
    {
    case SNMP_PDU_GET:
        if (local == NULL)
        {
            Obj->Syntax.LngInt = SNMP_INVALID;
            return SNMP_NOSUCHNAME;
        }
        data = (HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->Status;
        return SNMP_NOERROR;
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
        {
            Obj->Syntax.LngInt = SNMP_INVALID;
            return SNMP_NOSUCHNAME;
        }
        data = (HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->Status;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        if (local == NULL)
        {
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_CREATEREQUEST:
                if ((local = MibInsert(Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
                    return SNMP_GENERROR;
                if ((local->Data = DnpapMalloc(sizeof(HOST_CONTROL))) == NULL)
                    return SNMP_GENERROR;
                data = (HOST_CONTROL*) local->Data;
                if (HostCInit(data) == TRUE)
                {
                    data->Index = local->Index;
                    data->Status = SNMP_UNDERCREATION;
                    DnpapMessage(DMC_MESSAGE, HOST_CREATE, "host: collector %ld created", local->Index);
                    return SNMP_NOERROR;
                }
                DnpapFree(local->Data);
                MibRemove (Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE);
                return SNMP_GENERROR;
            default:
                return SNMP_NOSUCHNAME;
            }
        }
        data = (HOST_CONTROL*) local->Data;
        switch (data->Status)
        {
        case SNMP_UNDERCREATION:
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_VALID:
                if (HostCStart(data) == TRUE)
                {
                    data->Status = SNMP_VALID;
                    DnpapMessage(DMC_MESSAGE, HOST_START, "host: collector %ld active", local->Index);
                    return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            case SNMP_INVALID:
                if (HostCStop(data) == TRUE)
                {
                    DnpapFree(local->Data);
                    DnpapMessage(DMC_MESSAGE, HOST_DESTROY, "host: collector %ld destroyed", local->Index);
                    if (MibRemove (Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE) == TRUE)
                        return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            default:
                return SNMP_BADVALUE;
            }
        case SNMP_VALID:
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_VALID:
                return SNMP_NOERROR;
            case SNMP_INVALID:
                if (HostCStop(data) == TRUE)
                {
                    DnpapFree(local->Data);
                    DnpapMessage(DMC_MESSAGE, HOST_DESTROY, "host: collector %ld destroyed", local->Index);
                    if (MibRemove (Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE) == TRUE)
                        return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            default:
                return SNMP_BADVALUE;
            }
        }
    }
    return SNMP_GENERROR;
}


WORD hostAddress (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        memcpy (Obj->Syntax.BufChr, data->Address, Obj->SyntaxLen = HOST_SZEADDR);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostCreationOrder (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = HostSearchCreationOrder(local->Data, data);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostInPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostOutPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostInOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostOutOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostOutErrors (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutErrors;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostOutBroadcastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBroadcastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostOutMulticastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutMulticastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeAddress (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        memcpy (Obj->Syntax.BufChr, data->Address, Obj->SyntaxLen = HOST_SZEADDR);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeCreationOrder (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = Obj->Id[IdLen+1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeInPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeOutPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeInOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeOutOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeOutErrors (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutErrors;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeOutBroadcastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBroadcastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD hostTimeOutMulticastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = HostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutMulticastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


VOID HostPkt(LONG channelIndex, PROT_PKT *pkt)
{
MIB_LOCAL *hostinstance;
HOST_CONTROL *hostcontrol;

	for (hostinstance = HostControlInstance; hostinstance != NULL; hostinstance = hostinstance->Next)
	{
		hostcontrol = hostinstance->Data;
		if (hostcontrol->Status == SNMP_VALID && 
            hostcontrol->Channel == TRUE &&
            hostcontrol->Source[hostcontrol->SourceLen-1] == channelIndex)
			HostHandlePkt(hostcontrol, pkt);
	}
}
