/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#include <dnpap.h>
#include <agent.h>
#include <config.h>
#include <snmp.h>
#include <message.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys.h>
#include <trap.h>
#include <dlog.h>

#include "deventd.h"
#include "devente.h"
#include "deventc.h"
#include "deventu.h"
#include "devent.h"


static CHAR MODULE[] = "DnpapEventM";


static MIB_LOCAL*	RmonEventInstance = NULL;


static SHORT LogMaxNrLogs = 100;


static BOOLEAN  EventMibNext(SNMP_OBJECT* obj, MIB_LOCAL** local, WORD idlen, WORD indexsize);
static BOOLEAN  EventFirstLastLog(DnpapRmonEvent *event, LONG *firstlog, LONG *lastlog);
static BOOLEAN  EventLogInRange(LONG firstlog, LONG lastlog, LONG logindex);
static DnpapRmonLog* EventLogEntry(SNMP_OBJECT *obj, WORD idlen, MIB_LOCAL *local);



BOOLEAN DnpapEventMInit(VOID)
{
    MessageConfig(DNPAP_EVENT_ERROR, "DnpapEvent");
	ConfigGetShort("beholder.event.maxnrlogs", &LogMaxNrLogs);
	if (LogMaxNrLogs < 2)
    {
        DnpapMessage(DMC_WARNING, DEVENT_MAX, "event: beholder.event.maxnrlogs < 2, setting it to 2");
        LogMaxNrLogs = 2;
    }
	
	return TRUE;
}


BOOLEAN EventMibNext(SNMP_OBJECT* obj, MIB_LOCAL** local, WORD idlen, WORD indexsize)
{
LONG firstlog, lastlog;

    if (*local == NULL || indexsize < 1)
        return FALSE;

    if (obj->IdLen < idlen + indexsize)
    {
        if (indexsize == 2)
        {                          
            while (*local != NULL && EventFirstLastLog((DnpapRmonEvent*)(*local)->Data, &firstlog, &lastlog) == FALSE)
                *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->Id[idlen+1] = firstlog;
            obj->IdLen = idlen + 2;
        }
        else
        if (obj->IdLen == idlen)
        {
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + 1;
        }
        return TRUE;
    }
    else
    {
        switch (indexsize)
        {            
        case 2:
            if (EventFirstLastLog((DnpapRmonEvent*)(*local)->Data, &firstlog, &lastlog) == TRUE &&
                EventLogInRange(firstlog, lastlog, obj->Id[idlen+1]) == TRUE &&
                obj->Id[idlen+1] != lastlog)
            {
                obj->Id[idlen+1]++;
                if (obj->Id[idlen+1] == 0)
                    obj->Id[idlen+1] = 1;
                return TRUE;
            }
        case 1:
			/* only go to next collector if requested index was higher or equal */
			if (obj->Id[idlen] >= (*local)->Index)
	            *local = (*local)->Next;
            if (*local != NULL)
            {
                if (indexsize == 2)
                {                          
                    while (*local != NULL && EventFirstLastLog((DnpapRmonEvent*)(*local)->Data, &firstlog, &lastlog) == FALSE)
                        *local = (*local)->Next;
                    if (*local == NULL)
                        return FALSE;
                    obj->Id[idlen] = (*local)->Index;
                    obj->Id[idlen+1] = firstlog;
                    obj->IdLen = idlen + 2;
                }
                else
                {
                    obj->Id[idlen] = (*local)->Index;
                    obj->IdLen = idlen + 1;
                }
                return TRUE;
            }
            return FALSE;
        }
        return FALSE;
    }
    return FALSE;
}


BOOLEAN EventFirstLastLog(DnpapRmonEvent *event, LONG *firstlog, LONG *lastlog)
{
	if (event->NrLogs == 0)
	{
		*firstlog = *lastlog = 0;
		return FALSE;
	}
	if (event->LogBounced == TRUE && event->LogPos < event->NrLogs)
		*firstlog = event->Logs[event->LogPos].Index;
	else
		*firstlog = event->Logs[0].Index;
	*lastlog = event->Logs[(event->LogPos > 0) ? (event->LogPos-1) : (event->NrLogs-1)].Index;
	return TRUE;
}


BOOLEAN EventLogInRange(LONG firstlog, LONG lastlog, LONG logindex)
{
	if (firstlog <= lastlog)
		return (logindex >= firstlog && logindex <= lastlog);
	else
		return (logindex >= firstlog || logindex <= lastlog);
}


WORD DnpapEventIndex(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
			
	if ((local = MibRmon(obj, RmonEventInstance, idlen, 1)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
        if (EventMibNext(obj, &local, idlen, 1) == FALSE)
	    return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		obj->Syntax.LngInt = local->Index;
		return SNMP_NOERROR;
	case SNMP_PDU_SET:
		return SNMP_READONLY;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventDescription(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonEvent* data = NULL;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 1)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 1) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		data = (DnpapRmonEvent*)local->Data;
		memcpy(obj->Syntax.BufChr, data->Description, obj->SyntaxLen = data->DescrLen);
		return SNMP_NOERROR;
	case SNMP_PDU_SET:
		data = (DnpapRmonEvent*)local->Data;
        if (obj->SyntaxLen > RMONEVENT_SZEDESCR)
            return SNMP_BADVALUE;
        memcpy(data->Description, obj->Syntax.BufChr, data->DescrLen = obj->SyntaxLen);
		return SNMP_NOERROR;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventType(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonEvent* data = NULL;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 1)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 1) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		data = (DnpapRmonEvent*)local->Data;
        obj->Syntax.LngInt = data->Type;
		return SNMP_NOERROR;
	case SNMP_PDU_SET:
		data = (DnpapRmonEvent*)local->Data;
        if (obj->Syntax.LngInt < DEVENTTYPE_MIN || obj->Syntax.LngInt > DEVENTTYPE_MAX)
            return SNMP_BADVALUE;
        data->Type = obj->Syntax.LngInt;
        return SNMP_NOERROR;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventCommunity(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonEvent* data = NULL;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 1)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 1) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		data = (DnpapRmonEvent*)local->Data;
        memcpy(obj->Syntax.BufChr, data->Community, obj->SyntaxLen = data->CommLen);
		return SNMP_NOERROR;
	case SNMP_PDU_SET:
		data = (DnpapRmonEvent*)local->Data;
        if (obj->SyntaxLen > RMONEVENT_SZECOMM)
            return SNMP_BADVALUE;
        memcpy(data->Community, obj->Syntax.BufChr, data->CommLen = obj->SyntaxLen);
		return SNMP_NOERROR;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventLastTimeSent(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonEvent* data = NULL;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 1)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 1) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		data = (DnpapRmonEvent*)local->Data;
        obj->Syntax.LngUns = data->LastTimeSent;
		return SNMP_NOERROR;
	case SNMP_PDU_SET:
        return SNMP_READONLY;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventOwner(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonEvent* data = NULL;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 1)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 1) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		data = (DnpapRmonEvent*)local->Data;
		memcpy(obj->Syntax.BufChr, data->Owner, obj->SyntaxLen = data->OwnerLen);
		return SNMP_NOERROR;
	case SNMP_PDU_SET:
		data = (DnpapRmonEvent*)local->Data;
		if (data->Status != SNMP_UNDERCREATION)
			return SNMP_READONLY;
		memcpy(data->Owner, obj->Syntax.BufChr, data->OwnerLen = obj->SyntaxLen);
		return SNMP_NOERROR;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventStatus(SNMP_OBJECT* obj, WORD idlen)
{
MIB_LOCAL *local = NULL;
DnpapRmonEvent *data;

    local = MibRmon(obj, RmonEventInstance, idlen, 1);

    switch (obj->Request)
    {
        case SNMP_PDU_GET:
            if (local == NULL)
            {
                obj->Syntax.LngInt = SNMP_INVALID;
                return SNMP_NOSUCHNAME;
            }
            data = (DnpapRmonEvent*)local->Data;
            obj->Syntax.LngInt = data->Status;
            return SNMP_NOERROR;
        case SNMP_PDU_NEXT:
            if (EventMibNext(obj, &local, idlen, 1) == FALSE)
            {
                obj->Syntax.LngInt = SNMP_INVALID;
                return SNMP_NOSUCHNAME;
            }
            data = (DnpapRmonEvent*)local->Data;
            obj->Syntax.LngInt = data->Status;
            return SNMP_NOERROR;
        case SNMP_PDU_SET:
            if (local == NULL)
            {
                switch (obj->Syntax.LngInt)
                {
                    case SNMP_CREATEREQUEST:
                        if ((local = MibInsert(obj, &RmonEventInstance, idlen, 1)) == NULL)
                            return SNMP_GENERROR;
                        if ((local->Data = NewDnpapRmonEvent(1)) == NULL)
                            return SNMP_GENERROR;
                        data = (DnpapRmonEvent*) local->Data;
                        if (DnpapEventCInit(data) == TRUE)
                        {
                            data->Status = SNMP_UNDERCREATION;
                            return SNMP_NOERROR;
                        }
                        DelDnpapRmonEvent(local->Data);
                        MibRemove(obj, &RmonEventInstance, idlen, 1);
                        return SNMP_GENERROR;
                    default:
                        return SNMP_NOSUCHNAME;
                }
            }
            data = (DnpapRmonEvent*)local->Data;
            switch (data->Status)
            {
                case SNMP_UNDERCREATION:
                    switch (obj->Syntax.LngInt)
                    {
                        case SNMP_VALID:
                            if (DnpapEventCStart(data) == TRUE)
                            {
                                data->Status = SNMP_VALID;
                                return SNMP_NOERROR;
                            }
                            else
                                return SNMP_GENERROR;
                        case SNMP_INVALID:
                            if (DnpapEventCStop(data) == TRUE)
                            {
                                DelDnpapRmonEvent(local->Data);
                                if (MibRemove(obj, &RmonEventInstance, idlen, 1) == TRUE)
                                    return SNMP_NOERROR;
                            }
                            return SNMP_GENERROR;
                        default:
                            return SNMP_BADVALUE;
                    }
                case SNMP_VALID:
                    switch (obj->Syntax.LngInt)
                    {
                        case SNMP_INVALID:
                            if (DnpapEventCStop(data) == TRUE)
                            {
                                DelDnpapRmonEvent(local->Data);
                                if (MibRemove(obj, &RmonEventInstance, idlen, 1) == TRUE)
                                    return SNMP_NOERROR;
                            }
                            return SNMP_GENERROR;
                        case SNMP_VALID:
                            return SNMP_NOERROR;
                        default:
                            return SNMP_BADVALUE;
                    }
            }
    }
    return SNMP_GENERROR;
}


WORD DnpapEventLogEventIndex(SNMP_OBJECT *obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonLog *log;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 2)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 2) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		if ((log = EventLogEntry(obj, idlen, local)) == NULL)
			return SNMP_NOSUCHNAME;
		return DnpapLogEventIndex(obj, idlen, log);
	case SNMP_PDU_SET:
        return SNMP_READONLY;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventLogIndex(SNMP_OBJECT *obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonLog *log;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 2)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 2) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		if ((log = EventLogEntry(obj, idlen, local)) == NULL)
			return SNMP_NOSUCHNAME;
		return DnpapLogIndex(obj, idlen, log);
	case SNMP_PDU_SET:
        return SNMP_READONLY;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventLogTime(SNMP_OBJECT *obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonLog *log;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 2)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 2) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		if ((log = EventLogEntry(obj, idlen, local)) == NULL)
			return SNMP_NOSUCHNAME;
		return DnpapLogTime(obj, idlen, log);
	case SNMP_PDU_SET:
        return SNMP_READONLY;
	}
    return SNMP_GENERROR;
}


WORD DnpapEventLogDescription(SNMP_OBJECT *obj, WORD idlen)
{
MIB_LOCAL* local = NULL;
DnpapRmonLog *log;
			
    if ((local = MibRmon(obj, RmonEventInstance, idlen, 2)) == NULL)
		return SNMP_NOSUCHNAME;
		
	switch (obj->Request)
	{
	case SNMP_PDU_NEXT:
		if (EventMibNext(obj, &local, idlen, 2) == FALSE)
			return SNMP_NOSUCHNAME;
	case SNMP_PDU_GET:
		if ((log = EventLogEntry(obj, idlen, local)) == NULL)
			return SNMP_NOSUCHNAME;
		return DnpapLogDescription(obj, idlen, log);
	case SNMP_PDU_SET:
        return SNMP_READONLY;
	}
    return SNMP_GENERROR;
}


DnpapRmonLog* EventLogEntry(SNMP_OBJECT *obj, WORD idlen, MIB_LOCAL *local)
{
LONG firstlog, lastlog, logindex, realindex;
DnpapRmonEvent *event = (DnpapRmonEvent*)local->Data;

	logindex = obj->Id[idlen+1];
	
	if (EventFirstLastLog(event, &firstlog, &lastlog) == FALSE || EventLogInRange(firstlog, lastlog, logindex) == FALSE)
        return NULL;
	
	if (logindex >= firstlog)
		realindex = event->LogPos + logindex - firstlog;
	else
		realindex = event->LogPos - lastlog + logindex - 1;
		
	if (realindex >= event->NrLogs)
		realindex -= event->NrLogs;
	else
	if (realindex < 0)
		realindex += event->NrLogs;
		
	return &event->Logs[realindex];
}


BOOLEAN DnpapGenerateEvent(CHAR *generator, LONG eventindex, WORD code, WORD specific, SNMP_OBJECT *list, WORD listlen)
{
MIB_LOCAL *local;
DnpapRmonEvent *event;
DnpapRmonLog	  *ptr;

	if (eventindex == 0)
		return FALSE;
		
	for (local = RmonEventInstance; local != NULL; local = local->Next)
	{
		event = (DnpapRmonEvent*)local->Data;
		if (event->Status == SNMP_VALID && eventindex == local->Index)
		{
			DnpapEventGenerated(eventindex);

            event->LastTimeSent = SysTime();
			
			if (event->Type == DEVENT_LOG || event->Type == DEVENT_LOG_TRAP)
			{
				if (event->LogBounced == TRUE || event->NrLogs == LogMaxNrLogs ||
				    (ptr = ReNewDnpapRmonLog(event->Logs, (WORD)event->NrLogs+1, (WORD)event->NrLogs, (WORD)LogMaxNrLogs)) == NULL)
				{
					event->LogBounced = TRUE;
					if (event->NrLogs == 0)
					{
						DnpapMessage(DMC_ERROR, DEVENT_NOLOGS, "event: could not create any log entries");
						return FALSE;
					}
					if (event->LogPos == event->NrLogs)
						event->LogPos = 0;
				}
				else
				{
					event->Logs = ptr;
					event->NrLogs++;
				}
				event->Logs[event->LogPos].Index = event->LogIndex++;
                if (event->Logs[event->LogPos].Index == 0)
                    event->Logs[event->LogPos].Index = event->LogIndex++;
				event->Logs[event->LogPos].Time = SysTime();
				sprintf(event->Logs[event->LogPos].Description, "event description: %.*s, generator: %s, event index: %ld, trap: %s, specific: %s%n",
                        event->DescrLen, event->Description, generator, eventindex, SnmpTrap[code], TrapSpecific[specific], &event->Logs[event->LogPos].DescrLen);
				event->LogPos++;
			}
			if (event->Type == DEVENT_SNMP_TRAP || event->Type == DEVENT_LOG_TRAP)
				if (AgentSendTrap(event->Community, event->CommLen, code, specific, list, listlen) == FALSE)
                    DnpapMessage(DMC_WARNING, DEVENT_TRAPFAIL, "event: trap send failed");
			
			return TRUE;
		}
	}
	
	return FALSE;
}
