/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <mibsup.h>
#include <message.h>
#include <protocol.h>

#include "dfiltere.h"
#include "dfilterc.h"
#include "dfilter.h"


#define INDEXSIZE   1


static MIB_LOCAL    *filter = NULL;


static BOOLEAN RmonNext (SNMP_OBJECT *Obj, MIB_LOCAL **Local, WORD IdLen, WORD IdSze, DNPAP_FILTER **Elm);



BOOLEAN DnpapFilterMInit(VOID)
{
    MessageConfig(DNPAP_FILTER_ERROR, "DnpapFilter");
    return TRUE;
}


/*****************************************************************
** NAME:        RmonNext
** SYNOPSIS:    BOOLEAN RmonNext (SNMP_OBJECT *Obj,
**                                  MIB_LOCAL **Local, WORD IdLen,
**                                  WORD IdSze, VOID **Elm)
** PARAMETERS:  Obj: requested object
**              Local: local datastructure: a collector
**              IdLen: identifier length known to the MIB
**              IdSze: size part application of identifier
**                     including the collector index !!
**              Elm: pointer to object if tables are used
** DESCRIPTION: application specific RMON NEXT function.
**              only called after MibRmon() for next requests 
**              searches the next object in the collector
** REMARKS:     ONLY FOR INTERNAL USE
** RETURNS:     TRUE: object found
**                    OPTIONAL: Elm -> pointer to object in table
*******************************************************************/

static BOOLEAN RmonNext(SNMP_OBJECT *Obj, MIB_LOCAL **Local, WORD IdLen, WORD IdSze, DNPAP_FILTER **Elm)
{
    MIB_LOCAL    *local = *Local;

    if (local == NULL)
        return FALSE;
    if (Obj->IdLen == IdLen || local->Index > Obj->Id[IdLen])
    {
        /********************************************************************
        **  OPTIONAL: search function to find first obj in a table !! 
        **  if (not found)
        **      search next collector
        **  adjust object identifier to identifier first object
        **  Elm -> found table obj
        ********************************************************************/
        Obj->IdLen = IdLen + IdSze;
        Obj->Id[IdLen] = local->Index;
        return TRUE;
    }
    /********************************************************************
    **  OPTIONAL: search function to find next obj in a table !! 
    **  if (found)
    **      adjust object identifier to identifier next object
    **      Elm -> found table obj
    **      return TRUE
    ********************************************************************/
    *Local = local->Next;
    return (RmonNext(Obj, Local, IdLen, IdSze, Elm));
}


WORD DnpapFilterIndex(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL    *local = NULL;

    if ((local = MibRmon (Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return(SNMP_NOSUCHNAME);
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        return SNMP_READONLY;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterChannelIndex(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {           
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->Channel;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        data->Channel = Obj->Syntax.LngInt;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktDataOffset(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->PktDataOffset;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (Obj->Syntax.LngInt < 0)
            return SNMP_BADVALUE;
        data->PktDataOffset = Obj->Syntax.LngInt;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktData(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;
    BYTE        *datap, *maskp, *notp;
    WORD        i;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        memcpy(Obj->Syntax.BufChr, data->PktData, Obj->SyntaxLen = data->PktDataLen);
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (Obj->SyntaxLen > SNMP_SIZE_BUFCHR)
            return SNMP_BADVALUE;
        if ((datap = DnpapRealloc(data->PktData, Obj->SyntaxLen)) == NULL)
            return SNMP_GENERROR;
        if ((maskp = DnpapRealloc(data->PktDataMask, Obj->SyntaxLen)) == NULL)
        {
            DnpapFree(datap);
            return SNMP_GENERROR;
        }
        if ((notp = DnpapRealloc(data->PktDataNotMask, Obj->SyntaxLen)) == NULL)
        {
            DnpapFree(datap);
            DnpapFree(maskp);
            return SNMP_GENERROR;
        }
        data->PktData = datap;
        data->PktDataMask = maskp;
        data->PktDataNotMask = notp;
        for (i = data->PktDataMaskLen; i < Obj->SyntaxLen; i++)
            data->PktDataMask[i] = 0xff;
        if (Obj->SyntaxLen < data->PktDataMaskLen)
            data->PktDataMaskLen = Obj->SyntaxLen;
        for (i = data->PktDataNotMaskLen; i < Obj->SyntaxLen; i++)
            data->PktDataNotMask[i] = 0x00;
        if (Obj->SyntaxLen < data->PktDataNotMaskLen)
            data->PktDataNotMaskLen = Obj->SyntaxLen;
        memcpy(data->PktData, Obj->Syntax.BufChr, data->PktDataLen = Obj->SyntaxLen);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktDataMask(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;
    WORD        i;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        memcpy(Obj->Syntax.BufChr, data->PktDataMask, Obj->SyntaxLen = data->PktDataMaskLen);
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (Obj->SyntaxLen > SNMP_SIZE_BUFCHR || Obj->SyntaxLen > data->PktDataLen)
            return SNMP_BADVALUE;
        memcpy(data->PktDataMask, Obj->Syntax.BufChr, Obj->SyntaxLen);
        for (i = Obj->SyntaxLen; i < data->PktDataLen; i++)
            data->PktDataMask[i] = 0xff;
        data->PktDataMaskLen = Obj->SyntaxLen;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktDataNotMask(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;
    WORD        i;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        memcpy(Obj->Syntax.BufChr, data->PktDataNotMask, Obj->SyntaxLen = data->PktDataNotMaskLen);
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (Obj->SyntaxLen > SNMP_SIZE_BUFCHR || Obj->SyntaxLen > data->PktDataLen)
            return SNMP_BADVALUE;
        memcpy(data->PktDataNotMask, Obj->Syntax.BufChr, Obj->SyntaxLen);
        for (i = Obj->SyntaxLen; i < data->PktDataLen; i++)
            data->PktDataNotMask[i] = 0x00;
        data->PktDataNotMaskLen = Obj->SyntaxLen;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktStatus(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->PktStatus;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        /* This is not supported: can not be set */
        return SNMP_BADVALUE;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktStatusMask(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->PktStatusMask;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        /* This is not supported: can not be set */
        return SNMP_BADVALUE;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterPktStatusNotMask(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->PktStatusNotMask;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        /* This is not supported: can not be set */
        return SNMP_BADVALUE;
    }
    return SNMP_GENERROR;
}


WORD DnpapFilterOwner(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    if ((local = MibRmon(Obj, filter, IdLen, INDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_FILTER*) local->Data;
        memcpy (Obj->Syntax.BufChr, data->Owner, data->OwnerLen);
        Obj->SyntaxLen = data->OwnerLen;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (Obj->SyntaxLen > SNMP_SIZE_BUFCHR)
            return SNMP_TOOBIG;
        memcpy (data->Owner, Obj->Syntax.BufChr, Obj->SyntaxLen);
        data->OwnerLen = Obj->SyntaxLen;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}

WORD DnpapFilterStatus(SNMP_OBJECT *Obj, WORD IdLen)
{
    MIB_LOCAL   *local = NULL;
    DNPAP_FILTER      *data;

    local = MibRmon(Obj, filter, IdLen, INDEXSIZE);

    switch (Obj->Request)
    {
    case SNMP_PDU_GET:
        if (local == NULL)
        {
            Obj->Syntax.LngInt = SNMP_INVALID;
            return SNMP_NOSUCHNAME;
        }
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->Status;
        return SNMP_NOERROR;
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, INDEXSIZE, NULL) == FALSE)
        {
            Obj->Syntax.LngInt = SNMP_INVALID;
            return SNMP_NOSUCHNAME;
        }
        data = (DNPAP_FILTER*) local->Data;
        Obj->Syntax.LngInt = data->Status;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        if (local == NULL)
        {
            switch (Obj->Syntax.LngInt)
            {
                case SNMP_CREATEREQUEST:
                    if ((local = MibInsert(Obj, &filter, IdLen, INDEXSIZE)) == NULL)
                        return SNMP_GENERROR;
                    if ((local->Data = DnpapMalloc(sizeof(DNPAP_FILTER))) == NULL)
                        return SNMP_GENERROR;
                    data = (DNPAP_FILTER*) local->Data;
                    memset(data, 0, sizeof(DNPAP_FILTER));
                    if (DnpapFilterCInit(data) == TRUE)
                    {
                        data->Status = SNMP_UNDERCREATION;
                        DnpapMessage(DMC_MESSAGE, DFILTER_CREATE, "filter: collector %ld created", local->Index);
                        return SNMP_NOERROR;
                    }
                    DnpapFree(local->Data);
                    MibRemove (Obj, &filter, IdLen, INDEXSIZE);
                    return SNMP_GENERROR;
                default:
                    return SNMP_NOSUCHNAME;
            }
        }
        data = (DNPAP_FILTER*) local->Data;
        switch (data->Status)
        {
        case SNMP_UNDERCREATION:
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_VALID:
                if (DnpapFilterCStart(data) == TRUE)
                {
                    data->Status = SNMP_VALID;
                    DnpapMessage(DMC_MESSAGE, DFILTER_ACTIVE, "filter: collector %ld active", local->Index);
                    return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            case SNMP_INVALID:
                if (DnpapFilterCStop(data) == TRUE)
                {
                    DnpapFree(local->Data);
                    DnpapMessage(DMC_MESSAGE, DFILTER_DESTROY, "filter: collector %ld destroyed", local->Index);
                    if (MibRemove (Obj, &filter, IdLen, INDEXSIZE) == TRUE)
                        return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            default:
                return SNMP_BADVALUE;
            }
        case SNMP_VALID:
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_INVALID:
                if (DnpapFilterCStop(data) == TRUE)
                {
                    DnpapFree(local->Data);
                    DnpapMessage(DMC_MESSAGE, DFILTER_DESTROY, "filter: collector %ld destroyed", local->Index);
                    if (MibRemove (Obj, &filter, IdLen, INDEXSIZE) == TRUE)
                        return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            case SNMP_VALID:
                return SNMP_NOERROR;
            default:
                return SNMP_BADVALUE;
            }
        }
    }
    return SNMP_GENERROR;
}


BOOLEAN DnpapFilterPkt(LONG channelIndex, PROT_PKT *pkt)
{
MIB_LOCAL *local;
DNPAP_FILTER *data;
PROT_OBJ len =   {0, {1,5}};
PROT_OBJ packet= {0, {1,7}};
BYTE scratch;
WORD bi, fb, lb;
BOOLEAN rule2fail, rule3fail, filterfound;

    if (ProtGetField(pkt, &len) == FALSE ||
        ProtGetField(pkt, &packet) == FALSE) 
    {
        DnpapMessage(DMC_ERROR, DFILTER_BADPACKET,"filter: bad packet");
        return FALSE;
    }

    /*  All results of filters that apply are OR'ed. Actually lazy evaluation  */
    /*  is implemented: the first filter that matches returns TRUE.            */
    /*  In the heuristics below, a 'continue' is equivalent to a failing match,  */
    /*  and a 'break' is equivalent to a successful match.                       */
    filterfound = FALSE;
    for (local = filter; local != NULL; local = local->Next)
    {
        data = (DNPAP_FILTER*) local->Data;
        if (data->Status == SNMP_VALID && data->Channel == channelIndex)
        {
            filterfound = TRUE;
            if (data->PktDataLen == 0)
                continue;	/*  no data, no match  */
            if (data->PktDataLen + data->PktDataOffset > len.Syntax.LngInt)
                continue;       /*  rule 1 of packet matching  */
            fb = (WORD)data->PktDataOffset;
            lb = (WORD)data->PktDataOffset+data->PktDataLen;
            rule2fail = FALSE;
            if (data->PktDataNotMaskAllZero == TRUE)
                rule3fail = FALSE;
            else
                rule3fail = TRUE;
            for (bi = fb; bi < lb; bi++)
            {
                scratch = (((BYTE*)packet.Syntax.Ptr)[bi] ^ data->PktData[bi-fb]) & data->PktDataMask[bi-fb];
                if ((scratch & ~data->PktDataNotMask[bi-fb]) != 0)
                {
                    rule2fail = TRUE;
                    break;
                }
                if (rule3fail == TRUE && (scratch & data->PktDataNotMask[bi-fb]) != 0)
                    rule3fail = FALSE;
            }
            if (rule2fail == FALSE && rule3fail == FALSE)
            {
                return TRUE;
            }
            else
                continue;
        }
    }

    if (filterfound == TRUE)
        return FALSE;
    return TRUE;
}

