/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#include <dnpap.h>
#include <dmath.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <message.h>
#include <mibsup.h>
#include <mac.h>

#include "dhoste.h"
#include "dhostc.h"
#include "dhost.h"


#define CONTROLINDEXSIZE   		1
#define HOSTINDEXSIZE      		7
#define TIMEINDEXSIZE      		2
#define HOSTBUCKETINDEXSIZE     8
#define TIMEBUCKETINDEXSIZE     3


static MIB_LOCAL    *HostControlInstance = NULL;


static BOOLEAN RmonNext (SNMP_OBJECT *obj, MIB_LOCAL **local, WORD idlen, WORD indexsize, DNPAP_HOST **elem);

static DNPAP_HOST* HostGetFirstNext(DNPAP_HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first);
static DNPAP_HOST* HostBucketGetFirstNext(DNPAP_HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first);
static DNPAP_HOST* HostTimeBucketGetFirstNext(DNPAP_HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first);



BOOLEAN DnpapHostMInit(VOID)
{
    MessageConfig(DNPAP_HOST_ERROR, "DnpapHost");
    return DnpapHostConfigInit();
}


BOOLEAN RmonNext (SNMP_OBJECT *obj, MIB_LOCAL **local, WORD idlen, WORD indexsize, DNPAP_HOST **elem)
{
BOOLEAN first = FALSE;

    if (*local == NULL || indexsize < 1)
        return FALSE;

    if (obj->IdLen < idlen + indexsize)
    {
        switch (indexsize)
        {
        case HOSTBUCKETINDEXSIZE:
            while (*local != NULL && (*elem = HostBucketGetFirstNext((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + HOSTBUCKETINDEXSIZE;
        	break;
        case HOSTINDEXSIZE:
            while (*local != NULL && (*elem = HostGetFirstNext((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + HOSTINDEXSIZE;
            break;
        case TIMEBUCKETINDEXSIZE:
            while (*local != NULL && (*elem = HostTimeBucketGetFirstNext((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + TIMEBUCKETINDEXSIZE;
        	break;
        case TIMEINDEXSIZE:
            while (*local != NULL && ((DNPAP_HOST_CONTROL*)(*local)->Data)->TableSize == 0)
                *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->Id[idlen+1] = 1;
            obj->IdLen = idlen + TIMEINDEXSIZE;
            *elem = DnpapHostTimeSearch((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen);
            break;
        case CONTROLINDEXSIZE:
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + CONTROLINDEXSIZE;
            break;
        }
        return TRUE;
    }
    else
    {
        switch (indexsize)
        {            
        case HOSTBUCKETINDEXSIZE:
            while (*local != NULL && (*elem = HostBucketGetFirstNext((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + HOSTBUCKETINDEXSIZE;
        	break;
        case HOSTINDEXSIZE:
            while (*local != NULL && (*elem = HostGetFirstNext((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + HOSTINDEXSIZE;
            break;
        case TIMEBUCKETINDEXSIZE:
            while (*local != NULL && (*elem = HostTimeBucketGetFirstNext((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen, first)) == NULL)
            {
                first = TRUE;
                *local = (*local)->Next;
            }
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + TIMEBUCKETINDEXSIZE;
        	break;
        case TIMEINDEXSIZE:
            if (obj->Id[idlen+1] < ((DNPAP_HOST_CONTROL*)(*local)->Data)->TableSize)
            {
                obj->Id[idlen] = (*local)->Index;
                obj->Id[idlen+1]++;
                obj->IdLen = idlen + TIMEINDEXSIZE;
            	*elem = DnpapHostTimeSearch((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen);
                break;
            }
            *local = (*local)->Next;
            while (*local != NULL && ((DNPAP_HOST_CONTROL*)(*local)->Data)->TableSize == 0)
                *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->Id[idlen+1] = 1;
            obj->IdLen = idlen + TIMEINDEXSIZE;
            *elem = DnpapHostTimeSearch((DNPAP_HOST_CONTROL*)(*local)->Data, obj, idlen);
            break;
        case CONTROLINDEXSIZE:
			/* only go to next collector if requested index was higher or equal */
			if (obj->Id[idlen] >= (*local)->Index)
	            *local = (*local)->Next;
            if (*local == NULL)
                return FALSE;
            obj->Id[idlen] = (*local)->Index;
            obj->IdLen = idlen + CONTROLINDEXSIZE;
            break;
        }
        return TRUE;
    }
    return FALSE;
}


DNPAP_HOST *HostGetFirstNext(DNPAP_HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first)
{
DNPAP_HOST host, *p;
INT i, len;

    if (hostcontrol->TableSize == 0)
        return NULL;

    if (first == TRUE)
    {          
        p = hostcontrol->LexList;
        for (i = 0; i < HOST_SZEADDR; i++)
            obj->Id[idlen+1+i] = (LONG)p->Address[i];
        obj->IdLen = idlen + HOSTINDEXSIZE;
        return p;
    }

    len = obj->IdLen-idlen-1;
    if (len < 0)
        len = 0;

    for (i = 0; i < len; i++)
        host.Address[i] = (BYTE)obj->Id[idlen+1+i];

    p = DnpapHostHashSearch(hostcontrol, &host, len);

    if (len == HOST_SZEADDR && p != NULL)
        p = p->LexNext;

    if (p != NULL)
    {
        for (i = 0; i < HOST_SZEADDR; i++)
            obj->Id[idlen+1+i] = (LONG)p->Address[i];
        obj->IdLen = idlen + HOSTINDEXSIZE;
    }
    return p;
}


DNPAP_HOST* HostBucketGetFirstNext(DNPAP_HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first)
{
DNPAP_HOST host, *p;
INT i, len;

    if (hostcontrol->TableSize == 0 || hostcontrol->NrBuckets == 0)
        return NULL;

    if (first == TRUE)
    {          
        p = hostcontrol->LexList;
        for (i = 0; i < HOST_SZEADDR; i++)
            obj->Id[idlen+1+i] = (LONG)p->Address[i];
        obj->Id[idlen+HOST_SZEADDR+1] = 1;
        obj->IdLen = idlen + HOSTBUCKETINDEXSIZE;
        return p;
    }

    len = obj->IdLen-idlen-1;
    if (len < 0)
        len = 0;
                                            
    for (i = 0; i < IntMin(len, HOST_SZEADDR); i++)
        host.Address[i] = (BYTE)obj->Id[idlen+1+i];

    p = DnpapHostHashSearch(hostcontrol, &host, IntMin(len, HOST_SZEADDR));
                      
    if (p != NULL)
    {
        if (len == HOST_SZEADDR+1)
        {
            if (obj->Id[idlen+HOST_SZEADDR+1] < hostcontrol->NrBuckets)
                obj->Id[idlen+HOST_SZEADDR+1]++;
            else                                
            {
                if ((p = p->LexNext) != NULL)
                    obj->Id[idlen+HOST_SZEADDR+1] = 1;
            }
        }
        else
            obj->Id[idlen+HOST_SZEADDR+1] = 1;
    }

    if (p != NULL)
    {
        for (i = 0; i < HOST_SZEADDR; i++)
            obj->Id[idlen+1+i] = (LONG)p->Address[i];
        obj->IdLen = idlen + HOSTBUCKETINDEXSIZE;
    }
    return p;
}


DNPAP_HOST* HostTimeBucketGetFirstNext(DNPAP_HOST_CONTROL *hostcontrol, SNMP_OBJECT *obj, WORD idlen, BOOLEAN first)
{
INT len;

    if (hostcontrol->TableSize == 0 || hostcontrol->NrBuckets == 0)
        return NULL;

    if (first == TRUE)
    {          
        obj->Id[idlen+1] = 1;
        obj->Id[idlen+1+1] = 1;
        obj->IdLen = idlen + TIMEBUCKETINDEXSIZE;
        return DnpapHostTimeSearch(hostcontrol, obj, idlen);
    }

    len = obj->IdLen-idlen-1;
    if (len < 0)
        len = 0;

    switch (len)
    {
    case 0:
        obj->Id[idlen+1] = 1;
    case 1:
        obj->Id[idlen+1+1] = 1;
        break;
    case 2:
        if (obj->Id[idlen+1+1] < hostcontrol->NrBuckets)
            obj->Id[idlen+1+1]++;                      
        else
        {
            if (obj->Id[idlen+1] < hostcontrol->TableSize)
                obj->Id[idlen+1]++;
            else
                return NULL;    
            obj->Id[idlen+1+1] = 1;
        }
        break;
    }         
    
    obj->IdLen = idlen + TIMEBUCKETINDEXSIZE;
    
    return DnpapHostTimeSearch(hostcontrol, obj, idlen);
}


WORD DnpapHostControlIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlDataSource (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;
MAC_IFACE    *iface;
LONG         IfaceOid[] = {1,3,6,1,2,1,2,2,1,1,1};
WORD         IfaceOidLen = sizeof(IfaceOid)/sizeof(LONG);
LONG         ChannelOid[] = {1,3,6,1,2,1,16,7,2,1,1,1};
WORD         ChannelOidLen = sizeof(ChannelOid)/sizeof(LONG);

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {           
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        memcpy (Obj->Syntax.BufInt, data->Source, data->SourceLen*sizeof(data->Source[0]));
        Obj->SyntaxLen = data->SourceLen;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (IfaceOidLen == Obj->SyntaxLen &&
            memcmp(IfaceOid, Obj->Syntax.BufInt, (IfaceOidLen-1)*sizeof(IfaceOid[0])) == 0)
        {
            data->Channel = FALSE;
            if ((iface = MacIfaceGet((WORD) Obj->Syntax.BufInt[Obj->SyntaxLen-1])) == NULL)
                return SNMP_BADVALUE;
            if (iface->type != MAC_TYPE_ETHERNET_CSMACD && iface->type != MAC_TYPE_88023_CSMACD)
           	    return SNMP_BADVALUE;
            memcpy(data->Source, Obj->Syntax.BufInt, Obj->SyntaxLen*sizeof(data->Source[0]));
            data->SourceLen = Obj->SyntaxLen;
            data->Iface = iface;
            return SNMP_NOERROR;
        }
        else
        if (ChannelOidLen == Obj->SyntaxLen &&
            memcmp(ChannelOid, Obj->Syntax.BufInt, (ChannelOidLen-1)*sizeof(ChannelOid[0])) == 0)
        {                         
            data->Channel = TRUE;
            memcpy(data->Source, Obj->Syntax.BufInt, Obj->SyntaxLen*sizeof(data->Source[0]));
            data->SourceLen = Obj->SyntaxLen;
            return SNMP_NOERROR;
        }
        return SNMP_BADVALUE;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlTableSize (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->TableSize;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlLastDeleteTime (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        Obj->Syntax.LngUns = data->LastDeleteTime;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlBuckets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->NrBuckets;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (Obj->Syntax.LngInt < 0)
            return SNMP_BADVALUE;
        data->NrBuckets = Obj->Syntax.LngInt;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlInterval (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->Interval;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        if (Obj->Syntax.LngInt < 1)
            return SNMP_BADVALUE;
        data->Interval = Obj->Syntax.LngInt;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlOwner (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;

    if ((local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        memcpy (Obj->Syntax.BufChr, data->Owner, data->OwnerLen);
        Obj->SyntaxLen = data->OwnerLen;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        data = (DNPAP_HOST_CONTROL*) local->Data;
        if (data->Status != SNMP_UNDERCREATION)
            return SNMP_READONLY;
        memcpy (data->Owner, Obj->Syntax.BufChr, Obj->SyntaxLen);
        data->OwnerLen = Obj->SyntaxLen;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostControlStatus (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL    *local = NULL;
DNPAP_HOST_CONTROL *data;

    local = MibRmon(Obj, HostControlInstance, IdLen, CONTROLINDEXSIZE);

    switch (Obj->Request)
    {
    case SNMP_PDU_GET:
        if (local == NULL)
        {
            Obj->Syntax.LngInt = SNMP_INVALID;
            return SNMP_NOSUCHNAME;
        }
        data = (DNPAP_HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->Status;
        return SNMP_NOERROR;
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, CONTROLINDEXSIZE, NULL) == FALSE)
        {
            Obj->Syntax.LngInt = SNMP_INVALID;
            return SNMP_NOSUCHNAME;
        }
        data = (DNPAP_HOST_CONTROL*) local->Data;
        Obj->Syntax.LngInt = data->Status;
        return SNMP_NOERROR;
    case SNMP_PDU_SET:
        if (local == NULL)
        {
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_CREATEREQUEST:
                if ((local = MibInsert(Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE)) == NULL)
                    return SNMP_GENERROR;
                if ((local->Data = DnpapMalloc(sizeof(DNPAP_HOST_CONTROL))) == NULL)
                    return SNMP_GENERROR;
                data = (DNPAP_HOST_CONTROL*) local->Data;
                if (DnpapHostCInit(data) == TRUE)
                {
                    data->Index = local->Index;
                    data->Status = SNMP_UNDERCREATION;
                    DnpapMessage(DMC_MESSAGE, DHOST_CREATE, "dhost: collector %ld created", local->Index);
                    return SNMP_NOERROR;
                }
                DnpapFree(local->Data);
                MibRemove (Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE);
                return SNMP_GENERROR;
            default:
                return SNMP_NOSUCHNAME;
            }
        }
        data = (DNPAP_HOST_CONTROL*) local->Data;
        switch (data->Status)
        {
        case SNMP_UNDERCREATION:
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_VALID:
                if (DnpapHostCStart(data) == TRUE)
                {
                    data->Status = SNMP_VALID;
                    DnpapMessage(DMC_MESSAGE, DHOST_START, "dhost: collector %ld active", local->Index);
                    return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            case SNMP_INVALID:
                if (DnpapHostCStop(data) == TRUE)
                {
                    DnpapFree(local->Data);
                    DnpapMessage(DMC_MESSAGE, DHOST_DESTROY, "dhost: collector %ld destroyed", local->Index);
                    if (MibRemove (Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE) == TRUE)
                        return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            default:
                return SNMP_BADVALUE;
            }
        case SNMP_VALID:
            switch (Obj->Syntax.LngInt)
            {
            case SNMP_VALID:
                return SNMP_NOERROR;
            case SNMP_INVALID:
                if (DnpapHostCStop(data) == TRUE)
                {
                    DnpapFree(local->Data);
                    DnpapMessage(DMC_MESSAGE, DHOST_DESTROY, "dhost: collector %ld destroyed", local->Index);
                    if (MibRemove (Obj, &HostControlInstance, IdLen, CONTROLINDEXSIZE) == TRUE)
                        return SNMP_NOERROR;
                }
                return SNMP_GENERROR;
            default:
                return SNMP_BADVALUE;
            }
        }
    }
    return SNMP_GENERROR;
}


WORD DnpapHostAddress (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        memcpy (Obj->Syntax.BufChr, data->Address, Obj->SyntaxLen = HOST_SZEADDR);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostCreationOrder (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = DnpapHostSearchCreationOrder(local->Data, data);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutErrors (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutErrors;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutBroadcastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBroadcastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutMulticastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutMulticastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInTCPPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InTCPPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutTCPPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutTCPPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInOtherPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InOtherPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutOtherPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutOtherPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts64 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket1Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts65to127 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket2Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts128to255 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket3Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts256to511 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket4Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts512to1023 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket5Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostInPkts1024to1518 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket6Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts64 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket1Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts65to127 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket2Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts128to255 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket3Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts256to511 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket4Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts512to1023 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket5Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostOutPkts1024to1518 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket6Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTAddress (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        memcpy (Obj->Syntax.BufChr, data->Address, Obj->SyntaxLen = HOST_SZEADDR);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTInterval (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = Obj->Id[IdLen+HOST_SZEADDR+1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTCreationOrder (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = DnpapHostSearchCreationOrder(local->Data, data);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTIATPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->IATBuckets[Obj->Id[IdLen+HOST_SZEADDR+1]-1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTISTPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, HOSTBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, HOSTBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostHashSearch2(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->ISTBuckets[Obj->Id[IdLen+HOST_SZEADDR+1]-1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeAddress (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        memcpy (Obj->Syntax.BufChr, data->Address, Obj->SyntaxLen = HOST_SZEADDR);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeCreationOrder (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = Obj->Id[IdLen+1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutOctets (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutOctets;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutErrors (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutErrors;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutBroadcastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBroadcastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutMulticastPkts (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutMulticastPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInTCPPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InTCPPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutTCPPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutTCPPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInOtherPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InOtherPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutOtherPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutOtherPkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts64 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket1Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts65to127 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket2Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts128to255 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket3Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts256to511 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket4Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts512to1023 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket5Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeInPkts1024to1518 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->InBucket6Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts64 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket1Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts65to127 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket2Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts128to255 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket3Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts256to511 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket4Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts512to1023 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket5Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostTimeOutPkts1024to1518 (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->OutBucket6Pkts;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTTimeAddress (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        memcpy (Obj->Syntax.BufChr, data->Address, Obj->SyntaxLen = HOST_SZEADDR);
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTTimeInterval (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = Obj->Id[IdLen+1+1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTTimeCreationOrder (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = Obj->Id[IdLen+1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTTimeIndex (SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;

    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngInt = local->Index;
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTTimeIATPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->IATBuckets[Obj->Id[IdLen+1+1]-1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


WORD DnpapHostIFTTimeISTPkts(SNMP_OBJECT *Obj, WORD IdLen)
{
MIB_LOCAL  *local = NULL;
DNPAP_HOST       *data = NULL;

    if ((local = MibRmon (Obj, HostControlInstance, IdLen, TIMEBUCKETINDEXSIZE)) == NULL)
        return SNMP_NOSUCHNAME;
        
    switch (Obj->Request)
    {
    case SNMP_PDU_NEXT:
        if (RmonNext (Obj, &local, IdLen, TIMEBUCKETINDEXSIZE, &data) == FALSE)
            return SNMP_NOSUCHNAME;
    case SNMP_PDU_GET:
        if (data == NULL && (data = DnpapHostTimeSearch(local->Data, Obj, IdLen)) == NULL)
            return SNMP_NOSUCHNAME;
        Obj->Syntax.LngUns = data->ISTBuckets[Obj->Id[IdLen+1+1]-1];
        return SNMP_NOERROR;
    }
    return SNMP_GENERROR;
}


VOID DnpapHostPkt(LONG channelIndex, PROT_PKT *pkt)
{
MIB_LOCAL *hostinstance;
DNPAP_HOST_CONTROL *hostcontrol;

	for (hostinstance = HostControlInstance; hostinstance != NULL; hostinstance = hostinstance->Next)
	{
		hostcontrol = hostinstance->Data;
		if (hostcontrol->Status == SNMP_VALID && 
            hostcontrol->Channel == TRUE &&
            hostcontrol->Source[hostcontrol->SourceLen-1] == channelIndex)
			DnpapHostHandlePkt(hostcontrol, pkt);
	}
}
