/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <dnpap.h>
#include <message.h>
#include <mac.h>
#include <timer.h>
#include <protocol.h>

#include "dstate.h"
#include "dstatc.h"


static VOID StatTimerCallback(TIMER_DESCR *timer, ULONG now, VOID *param);
static BOOLEAN StatMacCallback(MAC_COLL *collector, PROT_PKT *pkt);



BOOLEAN DnpapStatCInit (DNPAP_ETHER_STATS *etherStats)
{
LONG source[] = {1,3,6,1,2,1,2,2,1,1,1};
    
    etherStats->SourceLen = sizeof(source)/sizeof(long);
    memcpy (etherStats->Source, source, sizeof(source));

    etherStats->Octets = 0; 
    etherStats->Pkts = 0; 
    etherStats->Pkts64Octets = 0; 
    etherStats->Pkts65to127Octets = 0; 
    etherStats->Pkts128to255Octets = 0; 
    etherStats->Pkts256to511Octets = 0;
    etherStats->Pkts512to1023Octets = 0;
    etherStats->Pkts1024to1518Octets = 0;

	etherStats->Time = 0;

	etherStats->Timer = NULL;
    etherStats->UtilInterval = 100;
    etherStats->UtilTime = 0;
    etherStats->UtilOctets = 0;
    etherStats->UtilPkts = 0;
    etherStats->Utilization = 0;

	etherStats->Buckets = 10;
	etherStats->Interval = 1000;

	if ((etherStats->BucketArray = NewBuckets(etherStats->Buckets)) == NULL)
		return FALSE;

    if ((etherStats->Iface =
        MacIfaceGet((WORD) etherStats->Source[etherStats->SourceLen-1])) == NULL)
    {
        DnpapMessage(DMC_ERROR, DSTAT_NETINIT, "detherstat: network initialization failed");
        return (FALSE);
    }

    return TRUE;
}

BOOLEAN DnpapStatCStart (DNPAP_ETHER_STATS *etherStats)
{
    etherStats->Coll.Rcve       = StatMacCallback;
    etherStats->Coll.specific   = etherStats;
    if (!MacCollRegister(&(etherStats->Coll)))
    {
        DnpapMessage(DMC_ERROR, DSTAT_NETINIT, "detherstat: network initialization failed");
        return (FALSE);
    }

    if ((etherStats->Timer = TimerRegister(StatTimerCallback, etherStats, etherStats->UtilInterval*10, TIMER_FOREVER, TIMER_TYPE_SKIP)) == NULL)
    {
        DnpapMessage(DMC_ERROR, DSTAT_TIMER, "history: can not register a timer");
        return FALSE;
    }

    MacStatistics(etherStats->Iface, &etherStats->BaseStat);

    etherStats->UtilTime = TimerNow();
    etherStats->UtilOctets = etherStats->BaseStat.Octets;
    etherStats->UtilPkts = etherStats->BaseStat.Pkts;

    return TRUE;
}

BOOLEAN DnpapStatCStop (DNPAP_ETHER_STATS *etherStats)
{
    TimerRemove(etherStats->Timer);
    MacCollRemove(&(etherStats->Coll));
    DelBuckets(etherStats->BucketArray);
    return TRUE;
}

VOID StatTimerCallback(TIMER_DESCR *timer, ULONG now, VOID *param)
{
ULONG dtime;
MAC_STAT netstat;
ULONG octets, pkts;
DNPAP_ETHER_STATS *etherStats = param;

    dtime = now - etherStats->UtilTime;
    etherStats->UtilTime = now;

    MacStatistics(etherStats->Iface, &netstat);

    octets = netstat.Octets - etherStats->UtilOctets;
    pkts = netstat.Pkts - etherStats->UtilPkts;

    etherStats->Utilization = (octets + pkts*24)*8/dtime;

    etherStats->UtilOctets = netstat.Octets;
    etherStats->UtilPkts = netstat.Pkts;

    return;
}

BOOLEAN StatMacCallback(MAC_COLL *collector, PROT_PKT *pkt)
{
PROT_OBJ Interface = {1, {1,2}};
PROT_OBJ Size = {1, {1,4}};
PROT_OBJ Time = {1, {1,6}};
ULONG size;
ULONG time;
LONG iat, iatindex;
DNPAP_ETHER_STATS *etherStats = collector->specific;

    if (ProtGetField(pkt,&Interface) == TRUE &&
        Interface.Syntax.LngInt == etherStats->Source[etherStats->SourceLen-1])
    {
        if (ProtGetField(pkt,&Size) == TRUE &&
        	ProtGetField(pkt,&Time) == TRUE)
        {
        	etherStats->Pkts++;
        	
            size = Size.Syntax.LngUns + 4L;
            etherStats->Octets += size;
            if (size == 64)
                etherStats->Pkts64Octets++;
            if (size >= 65)
            {
                if (size <= 127)
                    etherStats->Pkts65to127Octets++;
                else if (size <= 255)
                    etherStats->Pkts128to255Octets++; 
                else if (size <= 511)
                    etherStats->Pkts256to511Octets++;
                else if (size <= 1023)
                    etherStats->Pkts512to1023Octets++;
                else if (size <= 1518)
                    etherStats->Pkts1024to1518Octets++;
            }
            
			time = Time.Syntax.LngUns;
			if (etherStats->Time == 0)
				etherStats->Time = time;
			else
			{
	            iat = time - etherStats->Time;
	            etherStats->Time = time;
	            iatindex = iat/etherStats->Interval;
	            if (iatindex >= etherStats->Buckets)
	                iatindex = etherStats->Buckets-1;
	            etherStats->BucketArray[iatindex]++;
			}
        }
    }

    return TRUE;
}

ULONG *NewBuckets(LONG nrbuckets)
{            
ULONG *p;

	if ((p = DnpapMalloc(nrbuckets*sizeof(ULONG))) == NULL)
        return NULL;
    memset(p, 0, (size_t)nrbuckets*sizeof(ULONG));
    return p;
}

ULONG *ReNewBuckets(ULONG *buckets, LONG nrbuckets)
{
	return DnpapRealloc(buckets, nrbuckets*sizeof(ULONG));
}

VOID DelBuckets(ULONG *buckets)
{
	DnpapFree(buckets);
}
