/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#define INCL_BASE
#ifdef OS2
	#include <os2.h>
#endif /* OS2 */
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <maxmem.h>
#include <dnpap.h>
#include <message.h>
#include <timer.h>
#include <block.h>
#include <mac.h>
#ifdef OS2
	#include <snor.h>
#else
#include <fcntl.h>
#include <dkvm.h>
#ifdef SUNOS
	#include <nit.h>
#else
#ifdef ULTRIX
	#include <pf.h>
#endif
#endif
#endif
#include <agent.h>
#ifdef OS2
	#include <ether.h>
#endif
#include <ip.h>
#include <icmp.h>
#include <udp.h>
#ifdef OS2
	#include <pipe.h>
	#include <echod.h>
	#include <pingd.h>
#endif	
#include <snmp.h>
#include <mibsup.h>
#include <sys.h>
#include <maxmib.h>
#include <agent.h>
#include <info.h>

#include <stat.h>
#include <hist.h>
#include <protocol.h>
#include <capture.h>
#include <filter.h>
#include <channel.h>
#include <matrix.h>
#include <host.h>
#include <topn.h>
#include <alarm.h>
#include <event.h>
#include <config.h>
#ifdef OS2
	#include <soft.h>
	#include <loop.h>
#endif
#include <gen.h>
#include <reach.h>

#ifdef PERFORMANCE_TEST
#include <perf.h>
#include <perfm.h>
#endif

#include <register.h>
#include <statbuf.h>

#ifdef DISCOVERY
#include <discnod.h>
#include <disccon.h>
#endif

#ifdef PACKETTYPE
#include <ptype.h>
#endif

#ifdef DNPAP_RMON_EXTENSIONS
#include <dstat.h>
#include <dfilter.h>
#include <dchan.h>
#include <dcapt.h>
#include <dhost.h>
#include <dtopn.h>
#include <dmatrix.h>
#endif

#ifdef NEURO
#include <neuro.h>
#endif

#include "beholder.h"



#ifdef UNIX
	Kvm *kvm = NULL;
#endif
    
              
BYTE *trapComm = NULL;
USHORT trapCommLen = 0;


MIB_ELEMENT beholderObj[] =
{
    { {1,3,6,1,2,1,1,1}, 8,  sysDescr, SNMP_DISPLAYSTR, MIB_READ},
    { {1,3,6,1,2,1,1,2}, 8,  sysObjectID, SNMP_OBJECTID, MIB_READ},
    { {1,3,6,1,2,1,1,3}, 8,  sysUpTime, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,1,4}, 8,  sysContact, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,1,5}, 8,  sysName, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,1,6}, 8,  sysLocation, SNMP_DISPLAYSTR, MIB_READ},
    { {1,3,6,1,2,1,1,7}, 8,  sysServices, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,1},  11,  EtherStatsIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,2},  11,  EtherStatsDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,1,1,1,3},  11,  EtherStatsDropEvents, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,4},  11,  EtherStatsOctets, SNMP_COUNTER, MIB_READ}, 
    { {1,3,6,1,2,1,16,1,1,1,5},  11,  EtherStatsPkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,6},  11,  EtherStatsBroadcastPkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,7},  11,  EtherStatsMulticastPkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,8},  11,  EtherStatsCRCAlignErrors, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,9},  11,  EtherStatsUndersizePkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,10}, 11,  EtherStatsOversizePkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,11}, 11,  EtherStatsFragments, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,12}, 11,  EtherStatsJabbers, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,13}, 11,  EtherStatsCollisions, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,14}, 11,  EtherStatsPkts64Octets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,15}, 11,  EtherStatsPkts65to127Octets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,16}, 11,  EtherStatsPkts128to255Octets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,17}, 11,  EtherStatsPkts256to511Octets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,18}, 11,  EtherStatsPkts512to1023Octets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,19}, 11,  EtherStatsPkts1024to1518Octets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,1,1,1,20}, 11,  EtherStatsOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,1,1,1,21}, 11,  EtherStatsStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,2,1,1,1},  11,  HistoryControlIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,2,1,1,2},  11,  HistoryControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,2,1,1,3},  11,  HistoryControlBucketsRequested, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,2,1,1,4},  11,  HistoryControlBucketsGranted, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,2,1,1,5},  11,  HistoryControlInterval, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,2,1,1,6},  11,  HistoryControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,2,1,1,7},  11,  HistoryControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,2,2,1,1},  11,  EtherHistoryIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,2},  11,  EtherHistorySampleIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,3},  11,  EtherHistoryIntervalStart, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,4},  11,  EtherHistoryDropEvents, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,5},  11,  EtherHistoryOctets, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,6},  11,  EtherHistoryPkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,7},  11,  EtherHistoryBroadcastPkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,8},  11,  EtherHistoryMulticastPkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,9},  11,  EtherHistoryCRCAlignErrors, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,10}, 11,  EtherHistoryUndersizePkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,11}, 11,  EtherHistoryOversizePkts, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,12}, 11,  EtherHistoryFragments, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,13}, 11,  EtherHistoryJabbers, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,14}, 11,  EtherHistoryCollisions, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,2,2,1,15}, 11,  EtherHistoryUtilization, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,3,1,1,1},  11,  AlarmIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,3,1,1,2},  11,  AlarmInterval, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,3},  11,  AlarmVariable, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,4},  11,  AlarmSampleType, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,5},  11,  AlarmValue, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,3,1,1,6},  11,  AlarmStartupAlarm, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,7},  11,  AlarmRisingThreshold, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,8},  11,  AlarmFallingThreshold, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,9},  11,  AlarmRisingEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,10}, 11,  AlarmFallingEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,11}, 11,  AlarmOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,3,1,1,12}, 11,  AlarmStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,4,1,1,1},  11,  hostControlIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,1,1,2},  11,  hostControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,4,1,1,3},  11,  hostControlTableSize, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,1,1,4},  11,  hostControlLastDeleteTime, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,4,1,1,5},  11,  hostControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,4,1,1,6},  11,  hostControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,4,2,1,1},  11,  hostAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,2},  11,  hostCreationOrder, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,3},  11,  hostIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,4},  11,  hostInPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,5},  11,  hostOutPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,6},  11,  hostInOctets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,7},  11,  hostOutOctets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,8},  11,  hostOutErrors, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,9},  11,  hostOutBroadcastPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,2,1,10}, 11,  hostOutMulticastPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,1},  11,  hostTimeAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,2},  11,  hostTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,3},  11,  hostTimeIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,4},  11,  hostTimeInPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,5},  11,  hostTimeOutPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,6},  11,  hostTimeInOctets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,7},  11,  hostTimeOutOctets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,8},  11,  hostTimeOutErrors, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,9},  11,  hostTimeOutBroadcastPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,4,3,1,10}, 11,  hostTimeOutMulticastPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,5,1,1,1},  11,  HostTopNControlIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,5,1,1,2},  11,  HostTopNHostIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,5,1,1,3},  11,  HostTopNRateBase, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,5,1,1,4},  11,  HostTopNTimeRemaining, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,5,1,1,5},  11,  HostTopNDuration, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,5,1,1,6},  11,  HostTopNRequestedSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,5,1,1,7},  11,  HostTopNGrantedSize, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,5,1,1,8},  11,  HostTopNStartTime, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,5,1,1,9},  11,  HostTopNOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,5,1,1,10}, 11,  HostTopNStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,5,2,1,1},  11,  HostTopNReport, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,5,2,1,2},  11,  HostTopNIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,5,2,1,3},  11,  HostTopNAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,5,2,1,4},  11,  HostTopNRate, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,1,1,1},  11,  MatrixControlIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,1,1,2},  11,  MatrixControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,6,1,1,3},  11,  MatrixControlTableSize, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,1,1,4},  11,  MatrixControlLastDeleteTime, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,6,1,1,5},  11,  MatrixControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,6,1,1,6},  11,  MatrixControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,6,2,1,1},  11,  MatrixSDSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,6,2,1,2},  11,  MatrixSDDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,6,2,1,3},  11,  MatrixSDIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,2,1,4},  11,  MatrixSDPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,2,1,5},  11,  MatrixSDOctets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,2,1,6},  11,  MatrixSDErrors, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,3,1,1},  11,  MatrixDSSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,6,3,1,2},  11,  MatrixDSDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,6,3,1,3},  11,  MatrixDSIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,3,1,4},  11,  MatrixDSPkts, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,3,1,5},  11,  MatrixDSOctets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,6,3,1,6},  11,  MatrixDSErrors, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,7,1,1,1},  11,  filterIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,7,1,1,2},  11,  filterChannelIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,3},  11,  filterPktDataOffset, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,4},  11,  filterPktData, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,5},  11,  filterPktDataMask, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,6},  11,  filterPktDataNotMask, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,7},  11,  filterPktStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,8},  11,  filterPktStatusMask, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,9},  11,  filterPktStatusNotMask, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,10}, 11,  filterOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,1,1,11}, 11,  filterStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,1},  11,  ChannelIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,7,2,1,2},  11,  ChannelIfIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,3},  11,  ChannelAcceptType, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,4},  11,  ChannelDataControl, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,5},  11,  ChannelTurnOnEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,6},  11,  ChannelTurnOffEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,7},  11,  ChannelEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,8},  11,  ChannelEventStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,9},  11,  ChannelMatches, SNMP_COUNTER, MIB_READ},
    { {1,3,6,1,2,1,16,7,2,1,10}, 11,  ChannelDescription, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,11}, 11,  ChannelOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,7,2,1,12}, 11,  ChannelStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,1},  11,  bufferControlIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,1,1,2},  11,  bufferControlChannelIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,3},  11,  bufferControlFullStatus, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,1,1,4},  11,  bufferControlFullAction, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,5},  11,  bufferControlCaptureSliceSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,6},  11,  bufferControlDownloadSliceSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,7},  11,  bufferControlDownloadOffset, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,8},  11,  bufferControlMaxOctetsRequested, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,9},  11,  bufferControlMaxOctetsGranted, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,1,1,10}, 11,  bufferControlCapturedPackets, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,1,1,11}, 11,  bufferControlTurnOnTime, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,8,1,1,12}, 11,  bufferControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,1,1,13}, 11,  bufferControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,8,2,1,1},  11,  captureBufferControlIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,2,1,2},  11,  captureBufferIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,2,1,3},  11,  captureBufferPacketID, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,2,1,4},  11,  captureBufferPacketData, SNMP_OCTETSTR, MIB_READ},
    { {1,3,6,1,2,1,16,8,2,1,5},  11,  captureBufferPacketLength, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,2,1,6},  11,  captureBufferPacketTime, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,8,2,1,7},  11,  captureBufferPacketStatus, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,9,1,1,1},  11,  EventIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,9,1,1,2},  11,  EventDescription, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,9,1,1,3},  11,  EventType, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,9,1,1,4},  11,  EventCommunity, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,9,1,1,5},  11,  EventLastTimeSent, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,9,1,1,6},  11,  EventOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,9,1,1,7},  11,  EventStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {1,3,6,1,2,1,16,9,2,1,1},  11,  EventLogEventIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,9,2,1,2},  11,  EventLogIndex, SNMP_INTEGER, MIB_READ},
    { {1,3,6,1,2,1,16,9,2,1,3},  11,  EventLogTime, SNMP_TIMETICKS, MIB_READ},
    { {1,3,6,1,2,1,16,9,2,1,4},  11,  EventLogDescription, SNMP_DISPLAYSTR, MIB_READ},
    { {BTNG_MIB,1,1,1,1}, BTNG_MIBLEN + 4,  InfoIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,1,1,1,2}, BTNG_MIBLEN + 4,  InfoDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,1,1,1,3}, BTNG_MIBLEN + 4,  InfoLostPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,1,1,1,4}, BTNG_MIBLEN + 4,  InfoPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,1,1,1,5}, BTNG_MIBLEN + 4,  InfoDiscardedPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,1,1,1,6}, BTNG_MIBLEN + 4,  InfoTruncatedPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,1,1,1,7}, BTNG_MIBLEN + 4,  InfoBufferedPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,1,1,1,8}, BTNG_MIBLEN + 4,  InfoStackedPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,1,1,1,9}, BTNG_MIBLEN + 4,  InfoOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,1,1,1,10}, BTNG_MIBLEN + 4, InfoStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

#ifdef PACKETTYPE
    { {BTNG_MIB,17,1,1,1}, BTNG_MIBLEN + 4, PacketTypeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,17,1,1,2}, BTNG_MIBLEN + 4, PacketTypeDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,17,1,1,3}, BTNG_MIBLEN + 4, PacketTypeOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,17,1,1,4}, BTNG_MIBLEN + 4, PacketTypePkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,17,1,1,5}, BTNG_MIBLEN + 4, PacketTypeMatch, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,17,1,1,6}, BTNG_MIBLEN + 4, PacketTypeOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,17,1,1,7}, BTNG_MIBLEN + 4, PacketTypeStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
#endif

#ifdef DNPAP_RMON_EXTENSIONS
    { {DNPAP_RMON_MIB,1,3,1,1},  DNPAP_RMON_MIBLEN + 4,  StatBufControlIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,3,1,2},  DNPAP_RMON_MIBLEN + 4,  StatBufControlVariable, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,3,1,3},  DNPAP_RMON_MIBLEN + 4,  StatBufControlRequestedSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,3,1,4},  DNPAP_RMON_MIBLEN + 4,  StatBufControlGrantedSize, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,3,1,5},  DNPAP_RMON_MIBLEN + 4,  StatBufControlInterval, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,3,1,6},  DNPAP_RMON_MIBLEN + 4,  StatBufControlSampleType, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,3,1,7},  DNPAP_RMON_MIBLEN + 4,  StatBufControlFullStatus, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,3,1,8},  DNPAP_RMON_MIBLEN + 4,  StatBufControlFullAction, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,3,1,9},  DNPAP_RMON_MIBLEN + 4,  StatBufControlAverage, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,3,1,10}, DNPAP_RMON_MIBLEN + 4,  StatBufControlDeviation, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,3,1,11}, DNPAP_RMON_MIBLEN + 4,  StatBufControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,3,1,12}, DNPAP_RMON_MIBLEN + 4,  StatBufControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,4,1,1},  DNPAP_RMON_MIBLEN + 4,  StatBufLogIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,4,1,2},  DNPAP_RMON_MIBLEN + 4,  StatBufIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,4,1,3},  DNPAP_RMON_MIBLEN + 4,  StatBufTime, SNMP_TIMETICKS, MIB_READ},
    { {DNPAP_RMON_MIB,1,4,1,4},  DNPAP_RMON_MIBLEN + 4,  StatBufValue, SNMP_INTEGER, MIB_READ},
#endif	

#ifdef PERFORMANCE_TEST
    { {BTNG_MIB,2,1}, BTNG_MIBLEN + 2,  PerfDataControl, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,2,2}, BTNG_MIBLEN + 2,  PerfPkts, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,2,3}, BTNG_MIBLEN + 2,  PerfOctets, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,2,4}, BTNG_MIBLEN + 2,  PerfTimeTotal, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,2,5}, BTNG_MIBLEN + 2,  PerfTimeMax, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,2,6}, BTNG_MIBLEN + 2,  PerfTimeMin, SNMP_INTEGER, MIB_READ},
#endif

    { {BTNG_MIB,3,1}, BTNG_MIBLEN + 2,  memoryMax, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,3,2}, BTNG_MIBLEN + 2,  memoryChunk, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,3,3}, BTNG_MIBLEN + 2,  memoryUsed, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,4,1}, BTNG_MIBLEN + 2,  RegisterBeholderMessage, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,4,2}, BTNG_MIBLEN + 2,  RegisterBeholder, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},

#ifdef NEURO
    { {BTNG_MIB,7,1},      BTNG_MIBLEN + 2,  NeuroNrNetworks, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,2},      BTNG_MIBLEN + 2,  NeuroRunningNrNetworks, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,1},  BTNG_MIBLEN + 4,  NeuroIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,2},  BTNG_MIBLEN + 4,  NeuroNrLayers, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,3},  BTNG_MIBLEN + 4,  NeuroNetRefresh, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,4},  BTNG_MIBLEN + 4,  NeuroTrain, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,5},  BTNG_MIBLEN + 4,  NeuroRequestedSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,6},  BTNG_MIBLEN + 4,  NeuroGrantedSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,7},  BTNG_MIBLEN + 4,  NeuroBatchSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,8},  BTNG_MIBLEN + 4,  NeuroLearningRate, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,9},  BTNG_MIBLEN + 4,  NeuroMomentum, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,10}, BTNG_MIBLEN + 4,  NeuroTotalSteps, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,11}, BTNG_MIBLEN + 4,  NeuroTrainingSteps, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,12}, BTNG_MIBLEN + 4,  NeuroError, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,3,1,13}, BTNG_MIBLEN + 4,  NeuroOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,3,1,14}, BTNG_MIBLEN + 4,  NeuroStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,4,1,1},  BTNG_MIBLEN + 4,  NeuroLayerIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,4,1,2},  BTNG_MIBLEN + 4,  NeuroLayerIndexLayer, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,4,1,3},  BTNG_MIBLEN + 4,  NeuroNrNeurons, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,1},  BTNG_MIBLEN + 4,  NeuroLayerNeuronIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,5,1,2},  BTNG_MIBLEN + 4,  NeuroLayerNeuronIndexLayer, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,5,1,3},  BTNG_MIBLEN + 4,  NeuroLayerNeuronIndexNeuron, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,5,1,4},  BTNG_MIBLEN + 4,  NeuroDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,5},  BTNG_MIBLEN + 4,  NeuroDataSourceMax, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,6},  BTNG_MIBLEN + 4,  NeuroDataSourceMin, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,7},  BTNG_MIBLEN + 4,  NeuroDataSourceAverageMethod, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,8},  BTNG_MIBLEN + 4,  NeuroDataSourceAverageSteps, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,9},  BTNG_MIBLEN + 4,  NeuroDataSourceTrap, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,10}, BTNG_MIBLEN + 4,  NeuroDataSourceTrapComm, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,11}, BTNG_MIBLEN + 4,  NeuroDataSourceThresholdCheck, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,7,5,1,12}, BTNG_MIBLEN + 4,  NeuroDataSourceValue, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,5,1,13}, BTNG_MIBLEN + 4,  NeuroDataSourceOutputValue, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,6,1,1},  BTNG_MIBLEN + 4,  NeuroLayerNeuronValueIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,6,1,2},  BTNG_MIBLEN + 4,  NeuroLayerNeuronValueIndexLayer, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,6,1,3},  BTNG_MIBLEN + 4,  NeuroLayerNeuronValueIndexNeuron, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,6,1,4},  BTNG_MIBLEN + 4,  NeuroLayerNeuronValueIndexValue, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,7,6,1,5},  BTNG_MIBLEN + 4,  NeuroDataSourceThresholdValue, SNMP_INTEGER, MIB_READ | MIB_WRITE},
#endif

    { {BTNG_MIB,12,1,1,1},  BTNG_MIBLEN + 4, ReachIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,2},  BTNG_MIBLEN + 4, ReachIPAddress, SNMP_IPADDR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,3},  BTNG_MIBLEN + 4, ReachDelay, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,4},  BTNG_MIBLEN + 4, ReachTimeout, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,5},  BTNG_MIBLEN + 4, ReachValueSteps, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,6},  BTNG_MIBLEN + 4, ReachICMPWeight, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,7},  BTNG_MIBLEN + 4, ReachUDPWeight, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,8},  BTNG_MIBLEN + 4, ReachRemaining, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,9},  BTNG_MIBLEN + 4, ReachICMPReceived, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,12,1,1,10}, BTNG_MIBLEN + 4, ReachUDPReceived, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,12,1,1,11}, BTNG_MIBLEN + 4, ReachICMPDropped, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,12,1,1,12}, BTNG_MIBLEN + 4, ReachUDPDropped, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,12,1,1,13}, BTNG_MIBLEN + 4, ReachICMPTime, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,14}, BTNG_MIBLEN + 4, ReachUDPTime, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,15}, BTNG_MIBLEN + 4, ReachICMPTimeMin, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,16}, BTNG_MIBLEN + 4, ReachUDPTimeMin, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,17}, BTNG_MIBLEN + 4, ReachICMPTimeMax, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,18}, BTNG_MIBLEN + 4, ReachUDPTimeMax, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,19}, BTNG_MIBLEN + 4, ReachTotalICMPTime, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,20}, BTNG_MIBLEN + 4, ReachTotalUDPTime, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,21}, BTNG_MIBLEN + 4, ReachValue, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,12,1,1,22}, BTNG_MIBLEN + 4, ReachOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,12,1,1,23}, BTNG_MIBLEN + 4, ReachStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

#ifdef DISCOVERY
    { {BTNG_MIB,16,1,1,1}, BTNG_MIBLEN + 4,  DiscNodeControlIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,1,1,2}, BTNG_MIBLEN + 4,  DiscNodeControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,16,1,1,3}, BTNG_MIBLEN + 4,  DiscNodeControlTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,1,1,4}, BTNG_MIBLEN + 4,  DiscNodeControlEthTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,1,1,5}, BTNG_MIBLEN + 4,  DiscNodeControlIPTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,1,1,6}, BTNG_MIBLEN + 4,  DiscNodeControlUDPTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,1,1,7}, BTNG_MIBLEN + 4,  DiscNodeControlTCPTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,1,1,8}, BTNG_MIBLEN + 4,  DiscNodeControlLastDeleteTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,1,1,9}, BTNG_MIBLEN + 4,  DiscNodeControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,16,1,1,10}, BTNG_MIBLEN + 4, DiscNodeControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,16,2,1,1}, BTNG_MIBLEN + 4,  DiscNodeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,2,1,2}, BTNG_MIBLEN + 4,  DiscNodeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,2,1,3}, BTNG_MIBLEN + 4,  DiscNodeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,2,1,4}, BTNG_MIBLEN + 4,  DiscNodeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,2,1,5}, BTNG_MIBLEN + 4,  DiscNodeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,2,1,6}, BTNG_MIBLEN + 4,  DiscNodeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,2,1,7}, BTNG_MIBLEN + 4,  DiscNodeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,2,1,8}, BTNG_MIBLEN + 4,  DiscNodeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,3,1,1}, BTNG_MIBLEN + 4,  DiscNodeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,3,1,2}, BTNG_MIBLEN + 4,  DiscNodeIPAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,3,1,3}, BTNG_MIBLEN + 4,  DiscNodeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,3,1,4}, BTNG_MIBLEN + 4,  DiscNodeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,3,1,5}, BTNG_MIBLEN + 4,  DiscNodeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,3,1,6}, BTNG_MIBLEN + 4,  DiscNodeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,3,1,7}, BTNG_MIBLEN + 4,  DiscNodeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,3,1,8}, BTNG_MIBLEN + 4,  DiscNodeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,3,1,9}, BTNG_MIBLEN + 4,  DiscNodeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,4,1,1}, BTNG_MIBLEN + 4,  DiscNodeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,4,1,2}, BTNG_MIBLEN + 4,  DiscNodeIPAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,4,1,3}, BTNG_MIBLEN + 4,  DiscNodeUDPAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,4,1,4}, BTNG_MIBLEN + 4,  DiscNodeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,4,1,5}, BTNG_MIBLEN + 4,  DiscNodeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,4,1,6}, BTNG_MIBLEN + 4,  DiscNodeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,4,1,7}, BTNG_MIBLEN + 4,  DiscNodeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,4,1,8}, BTNG_MIBLEN + 4,  DiscNodeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,4,1,9}, BTNG_MIBLEN + 4,  DiscNodeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,4,1,10}, BTNG_MIBLEN + 4, DiscNodeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,5,1,1}, BTNG_MIBLEN + 4,  DiscNodeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,5,1,2}, BTNG_MIBLEN + 4,  DiscNodeIPAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,5,1,3}, BTNG_MIBLEN + 4,  DiscNodeTCPAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,5,1,4}, BTNG_MIBLEN + 4,  DiscNodeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,5,1,5}, BTNG_MIBLEN + 4,  DiscNodeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,5,1,6}, BTNG_MIBLEN + 4,  DiscNodeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,5,1,7}, BTNG_MIBLEN + 4,  DiscNodeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,5,1,8}, BTNG_MIBLEN + 4,  DiscNodeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,5,1,9}, BTNG_MIBLEN + 4,  DiscNodeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,5,1,10}, BTNG_MIBLEN + 4, DiscNodeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,62,1,1}, BTNG_MIBLEN + 4,  DiscNodeTimeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,62,1,2}, BTNG_MIBLEN + 4,  DiscNodeTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,62,1,3}, BTNG_MIBLEN + 4,  DiscNodeTimeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,62,1,4}, BTNG_MIBLEN + 4,  DiscNodeTimeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,62,1,5}, BTNG_MIBLEN + 4,  DiscNodeTimeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,62,1,6}, BTNG_MIBLEN + 4,  DiscNodeTimeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,62,1,7}, BTNG_MIBLEN + 4,  DiscNodeTimeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,62,1,8}, BTNG_MIBLEN + 4,  DiscNodeTimeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,63,1,1}, BTNG_MIBLEN + 4,  DiscNodeTimeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,63,1,2}, BTNG_MIBLEN + 4,  DiscNodeTimeIPAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,63,1,3}, BTNG_MIBLEN + 4,  DiscNodeTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,63,1,4}, BTNG_MIBLEN + 4,  DiscNodeTimeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,63,1,5}, BTNG_MIBLEN + 4,  DiscNodeTimeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,63,1,6}, BTNG_MIBLEN + 4,  DiscNodeTimeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,63,1,7}, BTNG_MIBLEN + 4,  DiscNodeTimeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,63,1,8}, BTNG_MIBLEN + 4,  DiscNodeTimeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,63,1,9}, BTNG_MIBLEN + 4,  DiscNodeTimeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,64,1,1}, BTNG_MIBLEN + 4,  DiscNodeTimeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,64,1,2}, BTNG_MIBLEN + 4,  DiscNodeTimeIPAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,64,1,3}, BTNG_MIBLEN + 4,  DiscNodeTimeUDPAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,64,1,4}, BTNG_MIBLEN + 4,  DiscNodeTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,64,1,5}, BTNG_MIBLEN + 4,  DiscNodeTimeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,64,1,6}, BTNG_MIBLEN + 4,  DiscNodeTimeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,64,1,7}, BTNG_MIBLEN + 4,  DiscNodeTimeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,64,1,8}, BTNG_MIBLEN + 4,  DiscNodeTimeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,64,1,9}, BTNG_MIBLEN + 4,  DiscNodeTimeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,64,1,10}, BTNG_MIBLEN + 4, DiscNodeTimeLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,65,1,1}, BTNG_MIBLEN + 4,  DiscNodeTimeEthAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,65,1,2}, BTNG_MIBLEN + 4,  DiscNodeTimeIPAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,65,1,3}, BTNG_MIBLEN + 4,  DiscNodeTimeTCPAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,65,1,4}, BTNG_MIBLEN + 4,  DiscNodeTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,65,1,5}, BTNG_MIBLEN + 4,  DiscNodeTimeIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,65,1,6}, BTNG_MIBLEN + 4,  DiscNodeTimeInPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,65,1,7}, BTNG_MIBLEN + 4,  DiscNodeTimeInOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,65,1,8}, BTNG_MIBLEN + 4,  DiscNodeTimeOutPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,65,1,9}, BTNG_MIBLEN + 4,  DiscNodeTimeOutOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,65,1,10}, BTNG_MIBLEN + 4, DiscNodeTimeLRUTime, SNMP_TIMETICKS, MIB_READ},

    { {BTNG_MIB,16,21,1,1}, BTNG_MIBLEN + 4,  DiscConnControlIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,21,1,2}, BTNG_MIBLEN + 4,  DiscConnControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,16,21,1,3}, BTNG_MIBLEN + 4,  DiscConnControlTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,21,1,4}, BTNG_MIBLEN + 4,  DiscConnControlEthTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,21,1,5}, BTNG_MIBLEN + 4,  DiscConnControlIPTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,21,1,6}, BTNG_MIBLEN + 4,  DiscConnControlUDPTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,21,1,7}, BTNG_MIBLEN + 4,  DiscConnControlTCPTableSize, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,21,1,8}, BTNG_MIBLEN + 4,  DiscConnControlLastDeleteTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,21,1,9}, BTNG_MIBLEN + 4,  DiscConnControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,16,21,1,10},BTNG_MIBLEN + 4,  DiscConnControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {BTNG_MIB,16,22,1,1}, BTNG_MIBLEN + 4,  DiscConnSDEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,22,1,2}, BTNG_MIBLEN + 4,  DiscConnSDEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,22,1,3}, BTNG_MIBLEN + 4,  DiscConnSDIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,22,1,4}, BTNG_MIBLEN + 4,  DiscConnSDPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,22,1,5}, BTNG_MIBLEN + 4,  DiscConnSDOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,22,1,6}, BTNG_MIBLEN + 4,  DiscConnSDLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,23,1,1}, BTNG_MIBLEN + 4,  DiscConnSDEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,23,1,2}, BTNG_MIBLEN + 4,  DiscConnSDIPSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,23,1,3}, BTNG_MIBLEN + 4,  DiscConnSDEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,23,1,4}, BTNG_MIBLEN + 4,  DiscConnSDIPDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,23,1,5}, BTNG_MIBLEN + 4,  DiscConnSDIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,23,1,6}, BTNG_MIBLEN + 4,  DiscConnSDPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,23,1,7}, BTNG_MIBLEN + 4,  DiscConnSDOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,23,1,8}, BTNG_MIBLEN + 4,  DiscConnSDLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,24,1,1}, BTNG_MIBLEN + 4,  DiscConnSDEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,24,1,2}, BTNG_MIBLEN + 4,  DiscConnSDIPSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,24,1,3}, BTNG_MIBLEN + 4,  DiscConnSDUDPSourceAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,24,1,4}, BTNG_MIBLEN + 4,  DiscConnSDEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,24,1,5}, BTNG_MIBLEN + 4,  DiscConnSDIPDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,24,1,6}, BTNG_MIBLEN + 4,  DiscConnSDUDPDestAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,24,1,7}, BTNG_MIBLEN + 4,  DiscConnSDIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,24,1,8}, BTNG_MIBLEN + 4,  DiscConnSDPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,24,1,9}, BTNG_MIBLEN + 4,  DiscConnSDOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,24,1,10}, BTNG_MIBLEN + 4, DiscConnSDLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,25,1,1}, BTNG_MIBLEN + 4,  DiscConnSDEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,25,1,2}, BTNG_MIBLEN + 4,  DiscConnSDIPSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,25,1,3}, BTNG_MIBLEN + 4,  DiscConnSDTCPSourceAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,25,1,4}, BTNG_MIBLEN + 4,  DiscConnSDEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,25,1,5}, BTNG_MIBLEN + 4,  DiscConnSDIPDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,25,1,6}, BTNG_MIBLEN + 4,  DiscConnSDTCPDestAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,25,1,7}, BTNG_MIBLEN + 4,  DiscConnSDIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,25,1,8}, BTNG_MIBLEN + 4,  DiscConnSDPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,25,1,9}, BTNG_MIBLEN + 4,  DiscConnSDOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,25,1,10}, BTNG_MIBLEN + 4, DiscConnSDLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,42,1,1}, BTNG_MIBLEN + 4,  DiscConnDSEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,42,1,2}, BTNG_MIBLEN + 4,  DiscConnDSEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,42,1,3}, BTNG_MIBLEN + 4,  DiscConnDSIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,42,1,4}, BTNG_MIBLEN + 4,  DiscConnDSPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,42,1,5}, BTNG_MIBLEN + 4,  DiscConnDSOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,42,1,6}, BTNG_MIBLEN + 4,  DiscConnDSLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,43,1,1}, BTNG_MIBLEN + 4,  DiscConnDSEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,43,1,2}, BTNG_MIBLEN + 4,  DiscConnDSIPSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,43,1,3}, BTNG_MIBLEN + 4,  DiscConnDSEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,43,1,4}, BTNG_MIBLEN + 4,  DiscConnDSIPDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,43,1,5}, BTNG_MIBLEN + 4,  DiscConnDSIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,43,1,6}, BTNG_MIBLEN + 4,  DiscConnDSPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,43,1,7}, BTNG_MIBLEN + 4,  DiscConnDSOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,43,1,8}, BTNG_MIBLEN + 4,  DiscConnDSLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,44,1,1}, BTNG_MIBLEN + 4,  DiscConnDSEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,44,1,2}, BTNG_MIBLEN + 4,  DiscConnDSIPSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,44,1,3}, BTNG_MIBLEN + 4,  DiscConnDSUDPSourceAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,44,1,4}, BTNG_MIBLEN + 4,  DiscConnDSEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,44,1,5}, BTNG_MIBLEN + 4,  DiscConnDSIPDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,44,1,6}, BTNG_MIBLEN + 4,  DiscConnDSUDPDestAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,44,1,7}, BTNG_MIBLEN + 4,  DiscConnDSIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,44,1,8}, BTNG_MIBLEN + 4,  DiscConnDSPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,44,1,9}, BTNG_MIBLEN + 4,  DiscConnDSOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,44,1,10}, BTNG_MIBLEN + 4, DiscConnDSLRUTime, SNMP_TIMETICKS, MIB_READ},
    { {BTNG_MIB,16,45,1,1}, BTNG_MIBLEN + 4,  DiscConnDSEthSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,45,1,2}, BTNG_MIBLEN + 4,  DiscConnDSIPSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,45,1,3}, BTNG_MIBLEN + 4,  DiscConnDSTCPSourceAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,45,1,4}, BTNG_MIBLEN + 4,  DiscConnDSEthDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,45,1,5}, BTNG_MIBLEN + 4,  DiscConnDSIPDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {BTNG_MIB,16,45,1,6}, BTNG_MIBLEN + 4,  DiscConnDSTCPDestAddress, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,45,1,7}, BTNG_MIBLEN + 4,  DiscConnDSIndex, SNMP_INTEGER, MIB_READ},
    { {BTNG_MIB,16,45,1,8}, BTNG_MIBLEN + 4,  DiscConnDSPkts, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,45,1,9}, BTNG_MIBLEN + 4,  DiscConnDSOctets, SNMP_COUNTER, MIB_READ},
    { {BTNG_MIB,16,45,1,10}, BTNG_MIBLEN + 4, DiscConnDSLRUTime, SNMP_TIMETICKS, MIB_READ},
#endif

#ifdef DNPAP_RMON_EXTENSIONS
    { {DNPAP_RMON_MIB,1,1,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,1,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsDropEvents, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsOctets, SNMP_COUNTER, MIB_READ}, 
    { {DNPAP_RMON_MIB,1,1,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsBroadcastPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsMulticastPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsCRCAlignErrors, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,9},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsUndersizePkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,10}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsOversizePkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,11}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsFragments, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,12}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsJabbers, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,13}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsInPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,14}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsInErrors, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,15}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsOutPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,16}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsOutErrors, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,17}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsCollisions, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,18}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts64, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,19}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts65to127, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,20}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts128to255, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,21}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts256to511, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,22}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts512to1023, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,23}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsPkts1024to1518, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,24}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsUtilInterval, SNMP_TIMETICKS, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,1,1,25}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsUtilization, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,1,1,26}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsBuckets, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,1,1,27}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsInterval, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,1,1,28}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,1,1,29}, DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,1,2,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsIFTInterval, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,2,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsIFTIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,1,2,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapEtherStatsIFTPkts, SNMP_COUNTER, MIB_READ},

    { {DNPAP_RMON_MIB,4,1,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,1,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,4,1,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlTableSize, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,1,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlLastDeleteTime, SNMP_TIMETICKS, MIB_READ},
    { {DNPAP_RMON_MIB,4,1,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlBuckets, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,4,1,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlInterval, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,4,1,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,4,1,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapHostControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

    { {DNPAP_RMON_MIB,4,2,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostCreationOrder, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapHostInOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapHostOutOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapHostOutErrors, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,9},  DNPAP_RMON_MIBLEN + 4,  DnpapHostOutBroadcastPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,10}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutMulticastPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,11}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts64, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,12}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts65to127, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,13}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts128to255, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,14}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts256to511, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,15}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts512to1023, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,16}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInPkts1024to1518, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,17}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts64, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,18}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts65to127, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,19}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts128to255, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,20}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts256to511, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,21}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts512to1023, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,22}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutPkts1024to1518, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,23}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInTCPPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,24}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutTCPPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,25}, DNPAP_RMON_MIBLEN + 4,  DnpapHostInOtherPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,2,1,26}, DNPAP_RMON_MIBLEN + 4,  DnpapHostOutOtherPkts, SNMP_COUNTER, MIB_READ},

    { {DNPAP_RMON_MIB,4,3,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutErrors, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,9},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutBroadcastPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,10}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutMulticastPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,11}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts64, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,12}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts65to127, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,13}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts128to255, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,14}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts256to511, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,15}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts512to1023, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,16}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInPkts1024to1518, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,17}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts64, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,18}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts65to127, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,19}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts128to255, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,20}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts256to511, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,21}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts512to1023, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,22}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutPkts1024to1518, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,23}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInTCPPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,24}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutTCPPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,25}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeInOtherPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,3,1,26}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTimeOutOtherPkts, SNMP_COUNTER, MIB_READ},

    { {DNPAP_RMON_MIB,4,4,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,4,4,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTInterval, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,4,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTCreationOrder, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,4,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,4,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTIATPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,4,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTISTPkts, SNMP_COUNTER, MIB_READ},

    { {DNPAP_RMON_MIB,4,5,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTTimeAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,4,5,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTTimeInterval, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,5,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTTimeCreationOrder, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,5,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTTimeIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,4,5,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTTimeIATPkts, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,4,5,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapHostIFTTimeISTPkts, SNMP_COUNTER, MIB_READ},

    { {DNPAP_RMON_MIB,5,1,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNControlIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,5,1,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNHostIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,5,1,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNRateBase, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,5,1,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNTimeRemaining, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,5,1,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNDuration, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,5,1,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNRequestedSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,5,1,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNGrantedSize, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,5,1,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNStartTime, SNMP_TIMETICKS, MIB_READ},
    { {DNPAP_RMON_MIB,5,1,1,9},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,5,1,1,10}, DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

    { {DNPAP_RMON_MIB,5,2,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNReport, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,5,2,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,5,2,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,5,2,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapHostTopNRate, SNMP_INTEGER, MIB_READ},

    { {DNPAP_RMON_MIB,6,1,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixControlIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,1,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixControlDataSource, SNMP_OBJECTID, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,6,1,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixControlTableSize, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,1,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixControlLastDelTime, SNMP_TIMETICKS, MIB_READ},
    { {DNPAP_RMON_MIB,6,1,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixControlOwner, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,6,1,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

    { {DNPAP_RMON_MIB,6,2,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDRanking, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDErrors, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,2,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixSDConnections, SNMP_INTEGER, MIB_READ},


    { {DNPAP_RMON_MIB,6,3,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSSourceAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSDestAddress, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSRanking, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSErrors, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,3,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapMatrixDSConnections, SNMP_INTEGER, MIB_READ},


    { {DNPAP_RMON_MIB,6,4,1,1},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNSource, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,2},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNDest, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,3},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,4},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNRanking, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,5},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,6},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,7},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNErrors, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,4,1,8},  DNPAP_RMON_MIBLEN + 4,  DMatrixSRankTopNConnections, SNMP_INTEGER, MIB_READ},


    { {DNPAP_RMON_MIB,6,5,1,1},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNSource, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,2},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNDest, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,3},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,4},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNRanking, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,5},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNPkts, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,6},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNOctets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,7},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNErrors, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,6,5,1,8},  DNPAP_RMON_MIBLEN + 4,  DMatrixDRankTopNConnections, SNMP_INTEGER, MIB_READ},


    { {DNPAP_RMON_MIB,7,1,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,7,1,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterChannelIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktDataOffset, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktData, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktDataMask, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktDataNotMask, SNMP_OCTETSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktStatusMask, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,9},  DNPAP_RMON_MIBLEN + 4,  DnpapFilterPktStatusNotMask, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,10}, DNPAP_RMON_MIBLEN + 4,  DnpapFilterOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,1,1,11}, DNPAP_RMON_MIBLEN + 4,  DnpapFilterStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

    { {DNPAP_RMON_MIB,7,2,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,7,2,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelIfIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelAcceptType, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelDataControl, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelTurnOnEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelTurnOffEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelEventIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,8},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelEventStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,9},  DNPAP_RMON_MIBLEN + 4,  DnpapChannelMatches, SNMP_COUNTER, MIB_READ},
    { {DNPAP_RMON_MIB,7,2,1,10}, DNPAP_RMON_MIBLEN + 4,  DnpapChannelDescription, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,11}, DNPAP_RMON_MIBLEN + 4,  DnpapChannelOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,7,2,1,12}, DNPAP_RMON_MIBLEN + 4,  DnpapChannelStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

    { {DNPAP_RMON_MIB,8,1,1,1},  DNPAP_RMON_MIBLEN + 4,  DBufferControlIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,1,1,2},  DNPAP_RMON_MIBLEN + 4,  DBufferControlChannelIndex, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,3},  DNPAP_RMON_MIBLEN + 4,  DBufferControlFullStatus, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,1,1,4},  DNPAP_RMON_MIBLEN + 4,  DBufferControlFullAction, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,5},  DNPAP_RMON_MIBLEN + 4,  DBufferControlCaptureSliceSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,6},  DNPAP_RMON_MIBLEN + 4,  DBufferControlDownloadSliceSize, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,7},  DNPAP_RMON_MIBLEN + 4,  DBufferControlDownloadOffset, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,8},  DNPAP_RMON_MIBLEN + 4,  DBufferControlMaxOctetsRqstd, SNMP_INTEGER, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,9},  DNPAP_RMON_MIBLEN + 4,  DBufferControlMaxOctetsGranted, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,1,1,10}, DNPAP_RMON_MIBLEN + 4,  DBufferControlCapturedPackets, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,1,1,11}, DNPAP_RMON_MIBLEN + 4,  DBufferControlTurnOnTime, SNMP_TIMETICKS, MIB_READ},
    { {DNPAP_RMON_MIB,8,1,1,12}, DNPAP_RMON_MIBLEN + 4,  DBufferControlOwner, SNMP_DISPLAYSTR, MIB_READ | MIB_WRITE},
    { {DNPAP_RMON_MIB,8,1,1,13}, DNPAP_RMON_MIBLEN + 4,  DBufferControlStatus, SNMP_INTEGER, MIB_READ | MIB_WRITE},

    { {DNPAP_RMON_MIB,8,2,1,1},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferControlIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,2,1,2},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferIndex, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,2,1,3},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferPacketID, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,2,1,4},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferPacketData, SNMP_OCTETSTR, MIB_READ},
    { {DNPAP_RMON_MIB,8,2,1,5},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferPacketLength, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,2,1,6},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferPacketTime, SNMP_INTEGER, MIB_READ},
    { {DNPAP_RMON_MIB,8,2,1,7},  DNPAP_RMON_MIBLEN + 4,  DnpapCaptureBufferPacketStatus, SNMP_INTEGER, MIB_READ},
#endif
};


static VOID BeholderSignalHandler(INT signum);



INT main(INT argc, char **argv)
{
    if (ConfigInit(argc, argv) == FALSE || ConfigLoad() == FALSE)
    {
        DnpapMessage(DMC_FATAL, BEHOLDER_CONFIGFAIL, "ConfigInit failed");
        DnpapExit(1);
    }

    MemoryInit(8000000L, 32000L);

    if (DnpapSignal(SIGINT, BeholderSignalHandler) == SIG_ERR)
        DnpapMessage(DMC_FATAL, BEHOLDER_SIGNALFAIL, "Signal handler initialization failed");
    
    MessageInit("stdio:out", MSGFILE, MSGSTDIO, NULL);
    MessageConfig(BEHOLDER_ERROR, "Beholder");
    
#ifdef UNIX
	if ((kvm = KvmOpen(O_RDONLY)) == NULL)
		DnpapMessage(DMC_FATAL, BEHOLDER_KVMFAIL, "Open of kernel memory failed");
#endif

    if (!TimerInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_TIMERFAIL,"TimerInit failed");

    /*  must come before PipeInit & SnorInit  */
#ifdef OS2
    if (!BlockInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKFAIL,"BlockInit failed");
#else	
    if (BlockInit() != 0)
        DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKFAIL,"BlockInit failed");
#endif		

    if (!ProtInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PROTFAIL,"ProtInit failed");
    
#ifdef OS2
    if (!PipeInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PIPEFAIL,"PipeInit failed");
#endif /* OS2 */

    /*  must come before MacInit  */
#ifdef OS2	
    if (!SnorInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"SnorInit failed");
#else
	if (!NwInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"NwInit failed");
#ifdef SUNOS
	if (!NitInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"NitInit failed");
#else
#ifdef ULTRIX
	if (!PfInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"PfInit failed");
#endif
#endif
#endif

    /*  must come before MacInit  */
#ifdef OS2	
    if (!LoopInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_LOOPFAIL,"LoopInit failed");
#endif /* OS2 */
    
    if (!MacInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_MACFAIL,"MacInit failed");

    /*  must come before MibInit  */
    if (!AgentInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_AGENTFAIL,"AgentInit failed");
		
    /*  must come after AgentInit  */
    if (!MibInit(beholderObj, sizeof(beholderObj)/sizeof(beholderObj[0])))
        DnpapMessage(DMC_WARNING,BEHOLDER_MIBFAIL,"MibInit failed");

#ifdef OS2    
    if (!ArpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ARPFAIL,"ArpInit failed");
    
    if (!IpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_IPFAIL,"IpInit failed");
#endif /* OS2 */
    
    if (!IcmpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ICMPFAIL,"IcmpInit failed");
    
    if (!UdpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_UDPFAIL,"UdpInit failed");
		
#ifdef OS2
    if (!EchodInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ECHOFAIL,"EchodInit failed");

    if (!PingdInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PINGFAIL,"PingdInit failed");
#endif /* OS2 */

#ifdef OS2
    if (!EtherInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ETHERFAIL,"EtherInit failed");
   
    if (!SoftInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SOFTFAIL,"SoftInit failed");
#endif /* OS2 */
     
    if (!GenInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_GENFAIL,"GenInit failed");
    
    if (!SysInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SYSFAIL,"SysInit failed");

    if (!AlarmMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ALARMFAIL,"AlarmMInit failed");
    
    if (!MatrixMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_MATRIXFAIL,"MatrixMInit failed");
    
    if (!HostMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_HOSTFAIL,"HostMInit failed");
    
    if (!TopNMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_TOPNFAIL,"TopNMInit failed");
    
    if (!ChannelMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_CHANNELFAIL,"ChannelMInit failed");
    
    if (!CaptureMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_CAPTUREFAIL,"CaptureMInit failed");
    
    if (!FilterMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_FILTERFAIL,"FilterMInit failed");

    /*  requests info on iface's, so place it at least after MacInit  */
    if (!EtherStatsMInit())   
        DnpapMessage(DMC_WARNING,BEHOLDER_STATSFAIL,"EtherStatsMInit failed");
    
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!HistoryMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_HISTORYFAIL,"HistoryMInit failed");
    
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!InfoMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_INFOFAIL,"InfoMInit failed");

#ifdef PACKETTYPE
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!PacketTypeMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_TYPEFAIL,"PacketTypeMInit failed");
#endif

#ifdef PERFORMANCE_TEST
    if (!PerfInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PERFFAIL,"PerfInit failed");
    
    if (!PerfMInit())   /*  MIB support initialization for perf  */
        DnpapMessage(DMC_WARNING,BEHOLDER_PERFMFAIL,"PerfMInit failed");
#endif
    
    if (!StatBufMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_STATBUFMFAIL,"StatBufMInit failed");

#ifdef NEURO
    if (!NeuroMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_NEUROMFAIL,"NeuroMInit failed");
#endif

    if (!ReachMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_REACHMFAIL,"ReachMInit failed");

#ifdef DISCOVERY   
    if (!DiscNodeMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DISCNODEMFAIL,"DiscNodeMInit failed");

    if (!DiscConnMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DISCCONNMFAIL,"DiscConnMInit failed");
#endif

#ifdef DNPAP_RMON_EXTENSIONS
    if (!DnpapEtherStatsMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPSTATSFAIL,"DnpapEtherStatsMInit failed");
    
    if (!DnpapFilterMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPFILTERFAIL,"DnpapFilterMInit failed");

    if (!DnpapCaptureMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPCAPTFAIL,"DnpapCaptureMInit failed");

    if (!DnpapHostMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPHOSTFAIL,"DnpapHostMInit failed");

    if (!DnpapTopNMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPTOPNFAIL,"DnpapTopNMInit failed");
    
    if (!DnpapMatrixMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPMATRIXFAIL,"DnpapMatrixMInit failed");
#endif

    if (ConfigGetBytes("beholder.trap.community", &trapComm, &trapCommLen) == FALSE)
	{
        trapComm = "trap";
        trapCommLen = 4;
	}
    if (!AgentSendTrap(trapComm, trapCommLen, SNMP_TRAP_COLDSTART, 0, 0, 0))
        DnpapMessage(DMC_WARNING,BEHOLDER_TRAPFAIL,"Could not send COLDSTART trap");

    DnpapMessage(DMC_MESSAGE,BEHOLDER_ACTIVE,"Beholder TNG active !\n");
 
#ifdef OS2    
    if (!Block())
        DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKERR,"Block exit !");
#else            
    while (1)
        if (Block()!=0)
            DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKERR,"Block exit !");
#endif
        
	DnpapExit(0);

    return 0;
}


VOID BeholderSignalHandler(INT signum)
{
#ifdef UNIX
	KvmClose(kvm);
#endif

    DnpapExit(0);
}

