/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <config.h>   
#include <mac.h>
#include <chain.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/dlpi.h>

#include "dlpi.h"
 

DLPI_DESCR *dlpiDescrList = 0;


static void    Rcve(NW_DG *nw, char *addr, BYTE *frame, int length, void *parm);
static BOOLEAN Stat(MAC_IFACE *iface, MAC_STAT *stats);
static BOOLEAN Send(MAC_IFACE *iface, CHAIN *chain);



void DlpiExit(void)
{
    DLPI_DESCR      *dlpi;
    
    while (dlpiDescrList!=0)
    {
        dlpi = dlpiDescrList;
        dlpiDescrList = dlpiDescrList->next;
                    
        MacIfaceRemove(dlpi->mac);
        NwDgClose(dlpi->nw);
        DnpapFree(dlpi);
    }
}


BOOLEAN DlpiInit(void)
{
    BYTE            name[255];
    BYTE            addr[255];
    CHAR            *descr;
    MAC_IFACE       *mac;
    DLPI_DESCR      *dlpi; 
    WORD            i;
    ULONG			offset;
    static BOOLEAN  init = FALSE;
    
    if (!init)
    {
        i = 1;
        sprintf(name, "dlpi.iface.name.%d", i);
        while (ConfigGetString(name, &descr))
        {
            mac = DnpapMalloc(sizeof(MAC_IFACE));
            if (mac!=0)
            {                        
                dlpi=DnpapMalloc(sizeof(DLPI_DESCR)); 
                memset(dlpi, 0, sizeof(DLPI_DESCR));
                if (dlpi!=0)
                {
                    dlpi->mac=mac;
                    
                    sprintf(addr,"dlpi:%s:8192:0", descr);
                    dlpi->nw=NwDgAccept(addr, Rcve, dlpi);  
                    if (dlpi->nw!=0)
                    {
                        mac->specific      = dlpi;
                        strcpy(mac->descr, descr);
                        mac->Send          = Send;
                        mac->Stat          = Stat;
                        mac->type          = MAC_TYPE_ETHERNET_CSMACD;
                        mac->speed         = 0;
                        mac->mtu           = DLPI_MTU;
                        mac->statusOper    = TRUE;
                        mac->statusAdmin   = TRUE;
                        mac->arp           = MAC_ARP_NONE;
                        memcpy(mac->addr, "\x00\x00\x00\x00\x00\x00",6);
                        memcpy(mac->addrBroadcast, "\xff\xff\xff\xff\xff\xff",6);
                        mac->addrLength    = 6;		 
						mac->frameId       = 0;
                        MacIfaceRegister(mac); 
                    }
                    
                    dlpi->next=dlpiDescrList;
                    dlpiDescrList=dlpi;
                }
            }    
            
            i++;
            sprintf(name, "dlpi.iface.name.%d", i);
        }
        
        DnpapAtExit(DlpiExit);

        init = TRUE;
    }

    return init;
}



static void Rcve(NW_DG *nw, char *addr, BYTE *frame, int length, void *parm)
{
    DLPI_DESCR          *dlpi;
    CHAIN               chain;
    MAC_INFO            info; 
    BYTE                *p;
        
    dlpi=(DLPI_DESCR *)parm;
    
    p=frame;
    
    info.copied      = length; 
    info.length      = ->sb_origlen;
    info.time        = ->sb_timestamp.tv_sec*1000000UL + ->sb_timestamp.tv_usec;
    info.promiscuous = ((p[0] & (BYTE)0x01)!=(BYTE)0x01 &&
                            memcmp(p, dlpi->mac->addr, 6)!=0);
    dlpi->Pkts++;
    dlpi->Octets += length;
    dlpi->DiscardedPkts = ->sb_drops;    
        
    if (info.length > info.copied)
        dlpi->TruncatedPkts++;
    if (!info.promiscuous)
        dlpi->StackedPkts++;
    if (memcmp(p,"\xff\xff\xff\xff\xff\xff",6)==0)
        dlpi->BroadcastPkts++;
    else if(p[0] & (BYTE)0x01)
        dlpi->MulticastPkts++;
    if (length < 60)
        dlpi->UndersizePkts++;           
    if (length > 1514)
        dlpi->OversizePkts++;           

    if (ChainAlloc(&chain, p, length, length, 0, 0)==0)
        return;
  
    MacRcve(dlpi->mac, &chain, &info);
}        


static BOOLEAN Stat(MAC_IFACE *iface, MAC_STAT *stats)
{
IFNET ifnet;

    DLPI_DESCR *dlpi;
    
    dlpi=(DLPI_DESCR *)iface->specific;

    stats->LostPkts         = dlpi->LostPkts      ;
    stats->Octets           = dlpi->Octets        ;
    stats->Pkts             = dlpi->Pkts          ;
    stats->BroadcastPkts    = dlpi->BroadcastPkts ;
    stats->MulticastPkts    = dlpi->MulticastPkts ;
    stats->CRCAlignErrors   = dlpi->CRCAlignErrors;         
    stats->UndersizePkts    = dlpi->UndersizePkts ;            
    stats->OversizePkts     = dlpi->OversizePkts  ;             
    stats->Fragments        = dlpi->Fragments     ;
    stats->Jabbers          = dlpi->Jabbers       ;         
    stats->Collisions       = dlpi->Collisions    ;
    stats->BufferedPkts     = dlpi->BufferedPkts  ;
    stats->StackedPkts      = dlpi->StackedPkts   ;
    stats->DiscardedPkts    = dlpi->DiscardedPkts ;
    stats->TruncatedPkts    = dlpi->TruncatedPkts ;
    return TRUE;
}


static BOOLEAN Send(MAC_IFACE *iface, CHAIN *chain)
{
    DLPI_DESCR  *dlpi;
    BYTE       frame[DLPI_MTU];
    WORD       length;
    BYTE       addr[32];
    
    dlpi=(DLPI_DESCR *)iface->specific;

    length = ChainLength(chain);
    if (!ChainCopy(chain, frame, length))
        return FALSE;
                   
    sprintf(addr,"dlpi:%s", iface->descr);
              
    if (!NwDgSendTo(0, addr, frame, length))
        return FALSE;

    return TRUE;
}
